/*----- PROTECTED REGION ID(TuneGof2.cpp) ENABLED START -----*/
/* clang-format on */
//=============================================================================
//
// file :        TuneGof2.cpp
//
// description : C++ source for the TuneGof2 class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               TuneGof2 are implemented in this file.
//
// project :     GOF2 Tune measurement
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include "TuneGof2.h"
#include "TuneGof2Class.h"


#include <fstream>
#include <sys/mman.h>
#include <fcntl.h> 
#include <sys/wait.h>
#include "tunethread.h"

#include <gof2_bpm_corr_tbl.h>
#include <gof2_fun.c>



/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2.cpp

/**
 *  TuneGof2 class description:
 *    
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name  |  Method name
//================================================================
//  State         |  Inherited (no method)
//  Status        |  Inherited (no method)
//  Start         |  start
//  Stop          |  stop
//  GetTune       |  get_tune
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  NumSamples           |  Tango::DevLong	Scalar
//  TuneMin              |  Tango::DevFloat	Scalar
//  TuneMax              |  Tango::DevFloat	Scalar
//  TuneMinPeakThresh    |  Tango::DevFloat	Scalar
//  Tune                 |  Tango::DevFloat	Scalar
//  MaxNumBpmUsed        |  Tango::DevLong	Scalar
//  LPCoeff              |  Tango::DevDouble	Scalar
//  TuneNotch            |  Tango::DevDouble	Scalar
//  TuneNotchWidth       |  Tango::DevDouble	Scalar
//  InhibitCycles        |  Tango::DevLong	Scalar
//  InhibitSegmentSize   |  Tango::DevLong	Scalar
//  DebugMinThres        |  Tango::DevDouble	Scalar
//  DebugMaxThres        |  Tango::DevDouble	Scalar
//  HistoryBufSize       |  Tango::DevLong	Scalar
//  EnableDespiking      |  Tango::DevBoolean	Scalar
//  DespikingFilterLen   |  Tango::DevLong	Scalar
//  Frequency            |  Tango::DevDouble	Scalar
//  FrequencySet         |  Tango::DevDouble	Scalar
//  InjTune              |  Tango::DevFloat	Scalar
//  InjPeakDetectThresh  |  Tango::DevDouble	Scalar
//  SumThres             |  Tango::DevULong	Scalar
//  EnableInjDespiking   |  Tango::DevBoolean	Scalar
//  TunePeakAmplitude    |  Tango::DevDouble	Scalar
//  FFTRaw               |  Tango::DevDouble	Spectrum  ( max = 4000000)
//  FFTFilt              |  Tango::DevDouble	Spectrum  ( max = 4000000)
//  BpmIndexList         |  Tango::DevDouble	Spectrum  ( max = 299)
//  TuneHistory          |  Tango::DevFloat	Spectrum  ( max = 1000000)
//  Debug                |  Tango::DevFloat	Spectrum  ( max = 2000000)
//================================================================

namespace TuneGof2_ns
{
/*----- PROTECTED REGION ID(TuneGof2::namespace_starting) ENABLED START -----*/
/* clang-format on */
//	static initializations
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::TuneGof2()
 * Description:  Constructors for a Tango device
 *                implementing the classTuneGof2
 */
//--------------------------------------------------------
TuneGof2::TuneGof2(Tango::DeviceClass *cl, std::string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(TuneGof2::constructor_1) ENABLED START -----*/
	/* clang-format on */
	init_device();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::constructor_1
}
//--------------------------------------------------------
TuneGof2::TuneGof2(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(TuneGof2::constructor_2) ENABLED START -----*/
	/* clang-format on */
	init_device();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::constructor_2
}
//--------------------------------------------------------
TuneGof2::TuneGof2(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(TuneGof2::constructor_3) ENABLED START -----*/
	/* clang-format on */
	init_device();
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::constructor_3
}
//--------------------------------------------------------
TuneGof2::~TuneGof2()
{
	delete_device();
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::delete_device()
 * Description:  will be called at device destruction or at init command
 */
//--------------------------------------------------------
void TuneGof2::delete_device()
{
	DEBUG_STREAM << "TuneGof2::delete_device() " << device_name << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::delete_device) ENABLED START -----*/
	/* clang-format on */
	//	Delete device allocated objects

	if (fft_mutex)
		delete fft_mutex;

	abort_tune_flag = true;
	while (run_tune_flag == true)
		usleep(10000);

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::delete_device
	delete[] attr_NumSamples_read;
	delete[] attr_TuneMin_read;
	delete[] attr_TuneMax_read;
	delete[] attr_TuneMinPeakThresh_read;
	delete[] attr_Tune_read;
	delete[] attr_MaxNumBpmUsed_read;
	delete[] attr_LPCoeff_read;
	delete[] attr_TuneNotch_read;
	delete[] attr_TuneNotchWidth_read;
	delete[] attr_InhibitCycles_read;
	delete[] attr_InhibitSegmentSize_read;
	delete[] attr_DebugMinThres_read;
	delete[] attr_DebugMaxThres_read;
	delete[] attr_HistoryBufSize_read;
	delete[] attr_EnableDespiking_read;
	delete[] attr_DespikingFilterLen_read;
	delete[] attr_Frequency_read;
	delete[] attr_FrequencySet_read;
	delete[] attr_InjTune_read;
	delete[] attr_InjPeakDetectThresh_read;
	delete[] attr_SumThres_read;
	delete[] attr_EnableInjDespiking_read;
	delete[] attr_TunePeakAmplitude_read;
	delete[] attr_FFTRaw_read;
	delete[] attr_FFTFilt_read;
	delete[] attr_BpmIndexList_read;
	delete[] attr_TuneHistory_read;
	delete[] attr_Debug_read;
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::init_device()
 * Description:  will be called at device initialization.
 */
//--------------------------------------------------------
void TuneGof2::init_device()
{
	DEBUG_STREAM << "TuneGof2::init_device() create device " << device_name << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::init_device_before) ENABLED START -----*/
	/* clang-format on */
	//	Initialization before get_device_property() call
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::init_device_before


	//	Get the device properties from database
	get_device_property();

	attr_NumSamples_read = new Tango::DevLong[1];
	attr_TuneMin_read = new Tango::DevFloat[1];
	attr_TuneMax_read = new Tango::DevFloat[1];
	attr_TuneMinPeakThresh_read = new Tango::DevFloat[1];
	attr_Tune_read = new Tango::DevFloat[1];
	attr_MaxNumBpmUsed_read = new Tango::DevLong[1];
	attr_LPCoeff_read = new Tango::DevDouble[1];
	attr_TuneNotch_read = new Tango::DevDouble[1];
	attr_TuneNotchWidth_read = new Tango::DevDouble[1];
	attr_InhibitCycles_read = new Tango::DevLong[1];
	attr_InhibitSegmentSize_read = new Tango::DevLong[1];
	attr_DebugMinThres_read = new Tango::DevDouble[1];
	attr_DebugMaxThres_read = new Tango::DevDouble[1];
	attr_HistoryBufSize_read = new Tango::DevLong[1];
	attr_EnableDespiking_read = new Tango::DevBoolean[1];
	attr_DespikingFilterLen_read = new Tango::DevLong[1];
	attr_Frequency_read = new Tango::DevDouble[1];
	attr_FrequencySet_read = new Tango::DevDouble[1];
	attr_InjTune_read = new Tango::DevFloat[1];
	attr_InjPeakDetectThresh_read = new Tango::DevDouble[1];
	attr_SumThres_read = new Tango::DevULong[1];
	attr_EnableInjDespiking_read = new Tango::DevBoolean[1];
	attr_TunePeakAmplitude_read = new Tango::DevDouble[1];
	attr_FFTRaw_read = new Tango::DevDouble[4000000];
	attr_FFTFilt_read = new Tango::DevDouble[4000000];
	attr_BpmIndexList_read = new Tango::DevDouble[299];
	attr_TuneHistory_read = new Tango::DevFloat[1000000];
	attr_Debug_read = new Tango::DevFloat[2000000];
	/*----- PROTECTED REGION ID(TuneGof2::init_device) ENABLED START -----*/
	/* clang-format on */
	//	Initialize device
	init_data();

	calc_compensation(*attr_NumSamples_read);

	if (hugePages)
		enable_hugepages = 1;
	else
		enable_hugepages = 0;	

	if (enable_hugepages)
		client_locate_shm(GOF2_SHM_HUGE_ID, &gof2, enable_hugepages);	
	else
		client_locate_shm(GOF2_SHM_ID, &gof2, enable_hugepages);

	fft_mutex = 0;
	fft_mutex = new omni_mutex();
	init_fft("");

	abort_tune_flag = false; run_tune_flag = false; err_tune_flag = false;

	set_state(Tango::OFF);
	set_status("Tune measurement OFF");

	if (autoStart)
		start();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::init_device
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::get_device_property()
 * Description:  Read database to initialize property data members.
 */
//--------------------------------------------------------
void TuneGof2::get_device_property()
{
	/*----- PROTECTED REGION ID(TuneGof2::get_device_property_before) ENABLED START -----*/
	/* clang-format on */
	//	Initialize property data members
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("HugePages"));
	dev_prop.push_back(Tango::DbDatum("SharedMemoryID"));
	dev_prop.push_back(Tango::DbDatum("BpmIndexList"));
	dev_prop.push_back(Tango::DbDatum("Plane"));
	dev_prop.push_back(Tango::DbDatum("AutoStart"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);

		//	get instance on TuneGof2Class to get class property
		Tango::DbDatum	def_prop, cl_prop;
		TuneGof2Class	*ds_class =
			(static_cast<TuneGof2Class *>(get_device_class()));
		int	i = -1;

		//	Try to initialize HugePages from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  hugePages;
		else {
			//	Try to initialize HugePages from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  hugePages;
		}
		//	And try to extract HugePages value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  hugePages;

		//	Try to initialize SharedMemoryID from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  sharedMemoryID;
		else {
			//	Try to initialize SharedMemoryID from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  sharedMemoryID;
		}
		//	And try to extract SharedMemoryID value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  sharedMemoryID;

		//	Try to initialize BpmIndexList from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  bpmIndexList;
		else {
			//	Try to initialize BpmIndexList from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  bpmIndexList;
		}
		//	And try to extract BpmIndexList value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  bpmIndexList;

		//	Try to initialize Plane from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  plane;
		else {
			//	Try to initialize Plane from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  plane;
		}
		//	And try to extract Plane value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  plane;

		//	Try to initialize AutoStart from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  autoStart;
		else {
			//	Try to initialize AutoStart from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  autoStart;
		}
		//	And try to extract AutoStart value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  autoStart;

	}

	/*----- PROTECTED REGION ID(TuneGof2::get_device_property_after) ENABLED START -----*/
	/* clang-format on */
	//	Check device property data members init
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::always_executed_hook()
 * Description:  method always executed before any command is executed
 */
//--------------------------------------------------------
void TuneGof2::always_executed_hook()
{
	DEBUG_STREAM << "TuneGof2::always_executed_hook()  " << device_name << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::always_executed_hook) ENABLED START -----*/
	/* clang-format on */
	//	code always executed before all requests

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::read_attr_hardware()
 * Description:  Hardware acquisition for attributes
 */
//--------------------------------------------------------
void TuneGof2::read_attr_hardware(TANGO_UNUSED(std::vector<long> &attr_list))
{
	DEBUG_STREAM << "TuneGof2::read_attr_hardware(std::vector<long> &attr_list) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_attr_hardware) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method      : TuneGof2::write_attr_hardware()
 * Description:  Hardware writing for attributes
 */
//--------------------------------------------------------
void TuneGof2::write_attr_hardware(TANGO_UNUSED(std::vector<long> &attr_list))
{
	DEBUG_STREAM << "TuneGof2::write_attr_hardware(std::vector<long> &attr_list) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::write_attr_hardware) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute NumSamples related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_NumSamples(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_NumSamples(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_NumSamples) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_NumSamples_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_NumSamples
}
//--------------------------------------------------------
/**
 *	Write attribute NumSamples related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_NumSamples(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_NumSamples(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_NumSamples) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_NumSamples_read = w_val;

	calc_compensation(*attr_NumSamples_read);

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_NumSamples
}
//--------------------------------------------------------
/**
 *	Read attribute TuneMin related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_TuneMin(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TuneMin(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TuneMin) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_TuneMin_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TuneMin
}
//--------------------------------------------------------
/**
 *	Write attribute TuneMin related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_TuneMin(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_TuneMin(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_TuneMin) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_TuneMin_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_TuneMin
}
//--------------------------------------------------------
/**
 *	Read attribute TuneMax related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_TuneMax(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TuneMax(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TuneMax) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_TuneMax_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TuneMax
}
//--------------------------------------------------------
/**
 *	Write attribute TuneMax related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_TuneMax(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_TuneMax(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_TuneMax) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_TuneMax_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_TuneMax
}
//--------------------------------------------------------
/**
 *	Read attribute TuneMinPeakThresh related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_TuneMinPeakThresh(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TuneMinPeakThresh(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TuneMinPeakThresh) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_TuneMinPeakThresh_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TuneMinPeakThresh
}
//--------------------------------------------------------
/**
 *	Write attribute TuneMinPeakThresh related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_TuneMinPeakThresh(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_TuneMinPeakThresh(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_TuneMinPeakThresh) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_TuneMinPeakThresh_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_TuneMinPeakThresh
}
//--------------------------------------------------------
/**
 *	Read attribute Tune related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_Tune(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_Tune(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_Tune) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_Tune_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_Tune
}
//--------------------------------------------------------
/**
 *	Read attribute MaxNumBpmUsed related method
 * Description:  Number of bpms used for tune estimation
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_MaxNumBpmUsed(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_MaxNumBpmUsed(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_MaxNumBpmUsed) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_MaxNumBpmUsed_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_MaxNumBpmUsed
}
//--------------------------------------------------------
/**
 *	Write attribute MaxNumBpmUsed related method
 * Description:  Number of bpms used for tune estimation
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_MaxNumBpmUsed(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_MaxNumBpmUsed(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_MaxNumBpmUsed) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_MaxNumBpmUsed
}
//--------------------------------------------------------
/**
 *	Read attribute LPCoeff related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_LPCoeff(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_LPCoeff(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_LPCoeff) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_LPCoeff_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_LPCoeff
}
//--------------------------------------------------------
/**
 *	Write attribute LPCoeff related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_LPCoeff(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_LPCoeff(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_LPCoeff) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_LPCoeff_read = w_val;

	calc_compensation(*attr_NumSamples_read);

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_LPCoeff
}
//--------------------------------------------------------
/**
 *	Read attribute TuneNotch related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_TuneNotch(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TuneNotch(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TuneNotch) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_TuneNotch_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TuneNotch
}
//--------------------------------------------------------
/**
 *	Write attribute TuneNotch related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_TuneNotch(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_TuneNotch(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_TuneNotch) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_TuneNotch_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_TuneNotch
}
//--------------------------------------------------------
/**
 *	Read attribute TuneNotchWidth related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_TuneNotchWidth(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TuneNotchWidth(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TuneNotchWidth) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_TuneNotchWidth_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TuneNotchWidth
}
//--------------------------------------------------------
/**
 *	Write attribute TuneNotchWidth related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_TuneNotchWidth(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_TuneNotchWidth(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_TuneNotchWidth) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_TuneNotchWidth_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_TuneNotchWidth
}
//--------------------------------------------------------
/**
 *	Read attribute InhibitCycles related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_InhibitCycles(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_InhibitCycles(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_InhibitCycles) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_InhibitCycles_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_InhibitCycles
}
//--------------------------------------------------------
/**
 *	Write attribute InhibitCycles related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_InhibitCycles(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_InhibitCycles(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_InhibitCycles) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_InhibitCycles_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_InhibitCycles
}
//--------------------------------------------------------
/**
 *	Read attribute InhibitSegmentSize related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_InhibitSegmentSize(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_InhibitSegmentSize(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_InhibitSegmentSize) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_InhibitSegmentSize_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_InhibitSegmentSize
}
//--------------------------------------------------------
/**
 *	Write attribute InhibitSegmentSize related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_InhibitSegmentSize(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_InhibitSegmentSize(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_InhibitSegmentSize) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_InhibitSegmentSize_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_InhibitSegmentSize
}
//--------------------------------------------------------
/**
 *	Read attribute DebugMinThres related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_DebugMinThres(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_DebugMinThres(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_DebugMinThres) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_DebugMinThres_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_DebugMinThres
}
//--------------------------------------------------------
/**
 *	Write attribute DebugMinThres related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_DebugMinThres(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_DebugMinThres(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_DebugMinThres) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_DebugMinThres_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_DebugMinThres
}
//--------------------------------------------------------
/**
 *	Read attribute DebugMaxThres related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_DebugMaxThres(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_DebugMaxThres(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_DebugMaxThres) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_DebugMaxThres_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_DebugMaxThres
}
//--------------------------------------------------------
/**
 *	Write attribute DebugMaxThres related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_DebugMaxThres(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_DebugMaxThres(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_DebugMaxThres) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_DebugMaxThres_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_DebugMaxThres
}
//--------------------------------------------------------
/**
 *	Read attribute HistoryBufSize related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_HistoryBufSize(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_HistoryBufSize(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_HistoryBufSize) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_HistoryBufSize_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_HistoryBufSize
}
//--------------------------------------------------------
/**
 *	Write attribute HistoryBufSize related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_HistoryBufSize(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_HistoryBufSize(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_HistoryBufSize) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_HistoryBufSize_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_HistoryBufSize
}
//--------------------------------------------------------
/**
 *	Read attribute EnableDespiking related method
 *
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_EnableDespiking(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_EnableDespiking(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_EnableDespiking) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_EnableDespiking_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_EnableDespiking
}
//--------------------------------------------------------
/**
 *	Write attribute EnableDespiking related method
 *
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_EnableDespiking(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_EnableDespiking(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_EnableDespiking) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_EnableDespiking_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_EnableDespiking
}
//--------------------------------------------------------
/**
 *	Read attribute DespikingFilterLen related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_DespikingFilterLen(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_DespikingFilterLen(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_DespikingFilterLen) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_DespikingFilterLen_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_DespikingFilterLen
}
//--------------------------------------------------------
/**
 *	Write attribute DespikingFilterLen related method
 *
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_DespikingFilterLen(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_DespikingFilterLen(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevLong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_DespikingFilterLen) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_DespikingFilterLen_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_DespikingFilterLen
}
//--------------------------------------------------------
/**
 *	Read attribute Frequency related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_Frequency(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_Frequency(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_Frequency) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_Frequency_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_Frequency
}
//--------------------------------------------------------
/**
 *	Read attribute FrequencySet related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_FrequencySet(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_FrequencySet(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_FrequencySet) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_FrequencySet_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_FrequencySet
}
//--------------------------------------------------------
/**
 *	Write attribute FrequencySet related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_FrequencySet(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_FrequencySet(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_FrequencySet) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_FrequencySet_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_FrequencySet
}
//--------------------------------------------------------
/**
 *	Read attribute InjTune related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_InjTune(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_InjTune(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_InjTune) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_InjTune_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_InjTune
}
//--------------------------------------------------------
/**
 *	Read attribute InjPeakDetectThresh related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_InjPeakDetectThresh(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_InjPeakDetectThresh(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_InjPeakDetectThresh) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_InjPeakDetectThresh_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_InjPeakDetectThresh
}
//--------------------------------------------------------
/**
 *	Write attribute InjPeakDetectThresh related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_InjPeakDetectThresh(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_InjPeakDetectThresh(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_InjPeakDetectThresh) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_InjPeakDetectThresh_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_InjPeakDetectThresh
}
//--------------------------------------------------------
/**
 *	Read attribute SumThres related method
 *
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_SumThres(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_SumThres(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_SumThres) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_SumThres_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_SumThres
}
//--------------------------------------------------------
/**
 *	Write attribute SumThres related method
 *
 *
 *	Data type:	Tango::DevULong
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_SumThres(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_SumThres(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevULong	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_SumThres) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_SumThres
}
//--------------------------------------------------------
/**
 *	Read attribute EnableInjDespiking related method
 *
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_EnableInjDespiking(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_EnableInjDespiking(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_EnableInjDespiking) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_EnableInjDespiking_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_EnableInjDespiking
}
//--------------------------------------------------------
/**
 *	Write attribute EnableInjDespiking related method
 *
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::write_EnableInjDespiking(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_EnableInjDespiking(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_EnableInjDespiking) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	*attr_EnableInjDespiking_read = w_val;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_EnableInjDespiking
}
//--------------------------------------------------------
/**
 *	Read attribute TunePeakAmplitude related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void TuneGof2::read_TunePeakAmplitude(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TunePeakAmplitude(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TunePeakAmplitude) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_TunePeakAmplitude_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TunePeakAmplitude
}
//--------------------------------------------------------
/**
 *	Read attribute FFTRaw related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 4000000
 */
//--------------------------------------------------------
void TuneGof2::read_FFTRaw(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_FFTRaw(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_FFTRaw) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(tune_fft_raw_data, *attr_NumSamples_read/2);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_FFTRaw
}
//--------------------------------------------------------
/**
 *	Read attribute FFTFilt related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 4000000
 */
//--------------------------------------------------------
void TuneGof2::read_FFTFilt(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_FFTFilt(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_FFTFilt) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(tune_fft_filt_data, *attr_NumSamples_read/2);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_FFTFilt
}
//--------------------------------------------------------
/**
 *	Read attribute BpmIndexList related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 299
 */
//--------------------------------------------------------
void TuneGof2::read_BpmIndexList(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_BpmIndexList(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_BpmIndexList) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	attr.set_value(attr_BpmIndexList_read, BpmIndexListAttrib::X_DATA_SIZE);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_BpmIndexList
}
//--------------------------------------------------------
/**
 *	Write attribute BpmIndexList related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 299
 */
//--------------------------------------------------------
void TuneGof2::write_BpmIndexList(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "TuneGof2::write_BpmIndexList(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve number of write values
	int	w_length = attr.get_write_value_length();

	//	Retrieve pointer on write values (Do not delete !)
	const Tango::DevDouble	*w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(TuneGof2::write_BpmIndexList) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::write_BpmIndexList
}
//--------------------------------------------------------
/**
 *	Read attribute TuneHistory related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Spectrum max = 1000000
 */
//--------------------------------------------------------
void TuneGof2::read_TuneHistory(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_TuneHistory(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_TuneHistory) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value

	for (int i = 0; i < *attr_HistoryBufSize_read; i++)
		attr_TuneHistory_read[i] = tune_circ_buf[(tune_buf_idx + i) % *attr_HistoryBufSize_read];

	attr.set_value(attr_TuneHistory_read, *attr_HistoryBufSize_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_TuneHistory
}
//--------------------------------------------------------
/**
 *	Read attribute Debug related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Spectrum max = 2000000
 */
//--------------------------------------------------------
void TuneGof2::read_Debug(Tango::Attribute &attr)
{
	DEBUG_STREAM << "TuneGof2::read_Debug(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::read_Debug) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	for (int i = 0; i < TUNEGOF2_DEBUGBUF_SIZE-1; i++)
		attr_Debug_read[i] = debug_circ_buf[(debug_buf_idx + 1 + i) % TUNEGOF2_DEBUGBUF_SIZE];


	attr.set_value(attr_Debug_read, TUNEGOF2_DEBUGBUF_SIZE-1);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::read_Debug
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2::add_dynamic_attributes()
 * Description:  Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void TuneGof2::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(TuneGof2::add_dynamic_attributes) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code to create and add dynamic attributes if any
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command Start related method
 *
 *
 */
//--------------------------------------------------------
void TuneGof2::start()
{
	DEBUG_STREAM << "TuneGof2::Start()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::start) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	run_tune_flag = false;
	abort_tune_flag = false;
	tuneloop = new tunethread(this);
	//rmloop->start_undetached();	
	tuneloop->start();

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::start
}
//--------------------------------------------------------
/**
 *	Command Stop related method
 *
 *
 */
//--------------------------------------------------------
void TuneGof2::stop()
{
	DEBUG_STREAM << "TuneGof2::Stop()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::stop) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code
	abort_tune_flag = true;
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::stop
}
//--------------------------------------------------------
/**
 *	Command GetTune related method
 *
 *
 *	@param argin mode(0);num samples
 *	@returns
 */
//--------------------------------------------------------
Tango::DevVarFloatArray *TuneGof2::get_tune(const Tango::DevVarLongArray *argin)
{
	Tango::DevVarFloatArray *argout;
	DEBUG_STREAM << "TuneGof2::GetTune()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(TuneGof2::get_tune) ENABLED START -----*/
	/* clang-format on */

	//	Add your own code

	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::get_tune
	return argout;
}
//--------------------------------------------------------
/**
 *	Method      : TuneGof2::add_dynamic_commands()
 * Description:  Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void TuneGof2::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(TuneGof2::add_dynamic_commands) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code to create and add dynamic commands if any
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2::add_dynamic_commands
}

/*----- PROTECTED REGION ID(TuneGof2::namespace_ending) ENABLED START -----*/
/* clang-format on */
//	Additional Methods


void TuneGof2::allocate_shm(void)
{

	fd_shm = shm_open(sharedMemoryID.c_str(), O_RDWR, S_IRUSR | S_IWUSR);
	
	if (fd_shm == -1)
	{
		perror("open");
	}

	// map shared memory to process address space
	gof2 = (struct gof2_shm_t *) mmap(NULL, sizeof(struct gof2_shm_t) , PROT_READ | PROT_WRITE, MAP_SHARED, fd_shm, 0);
	if (gof2 == MAP_FAILED)
	{
		perror("mmap");
	}

}


void TuneGof2::filtfilt(double *datain, double *dataout, double *datatmp, int num_samples)
{
	// calculate LP filter coefficients (Butterworth first order)
	double a = 1 - *attr_LPCoeff_read;
	double b = *attr_LPCoeff_read / 2;
 
	double max_datain = 0;
	for (int i = 0; i < num_samples; i++) {
		if (datain[i] > max_datain)
			max_datain = datain[i];
	}

	if (max_datain > TUNEOGF2_RESCALE) {
		for (int i = 0; i < num_samples; i++) {
			datain[i] = datain[i] * TUNEOGF2_RESCALE / max_datain;
		}
	}
	

	// direct low pass filter
	double x_old = 0, y_old = 0;
	for (int i = 0; i < num_samples; i++) {
		dataout[i] = a * y_old + datain[i] * b + x_old * b;
		x_old = datain[i];
		y_old = dataout[i];
	}	

	// inverted low pass filter
	x_old = 0; y_old = 0;
	for (int i = 0; i < num_samples; i++) {
		datatmp[i] = a * y_old + datain[num_samples-i-1] * b + x_old * b; // datain filipped
		x_old = datain[num_samples-i-1];
		y_old = datatmp[i];
	}

	// revert direction and compensate
	for (int i = 0; i < num_samples; i++) {
		dataout[i] += datatmp[num_samples-i-1]; // datatmp flipped left/right
		dataout[i] /= filter_compensation_vector[i];
	}

}


void TuneGof2::calc_compensation(int num_samples)
{
	// calculate LP filter coefficients (Butterworth first order)
	// for step response compensation
	double a = 1 - *attr_LPCoeff_read;
	double b = *attr_LPCoeff_read / 2;

	// step response of the filter	
	double x_old = 0, y_old = 0;
	for (int i = 0; i < num_samples; i++) {
		filter_compensation_vector[i] = a * y_old + 1 * b + x_old * b;
		x_old = 1;
		y_old = filter_compensation_vector[i];
	}

	// the first half compensation vector is equal to the average between the first half and the 
	// mirrored second half
	for (int i = 0; i < num_samples/2; i++) {
		filter_compensation_vector[i] += filter_compensation_vector[num_samples-i-1];
		filter_compensation_vector[i] /= 2;
	}	
	// then the second half is the mirrored first half
	// 0.1 0.2 0.9 1 1 1 1 1 0.9 0.2 0.1
	for (int i = 0; i < num_samples/2; i++) {
		filter_compensation_vector[num_samples/2+i] = filter_compensation_vector[num_samples/2-i-1]; 
	}
	// then the compensation vector is inverted
	for (int i = 0; i < num_samples; i++) {
		filter_compensation_vector[i] = 1 / filter_compensation_vector[i];
	}

}


void TuneGof2::init_data(void)
{

	for (int j = 0; j < TUNEGOF2_MAX_NUM_BPM; j++) {
		for (int k = 0; k < GOF2_BUF_SIZE; k++) {		
			gof2_data_in[j][k] = 0;
			tune_fft_bpm_data[j][k] = 0;
		}
	}


	for (int j = 0; j < GOF2_BUF_SIZE; j++) {
		tune_fft_raw_data[j] = 0;
		tune_fft_filt_data[j] = 0;
		filtbuf_tmp[j] = 0;
		filter_compensation_vector[j] = 0;
	}

	for (int j = 0; j < TUNEGOF2_CIRCBUF_SIZE; j++) {
		tune_circ_buf[j] = 0;
	}

	for (int j = 0; j < TUNEGOF2_DEBUGBUF_SIZE; j++) {
		debug_circ_buf[j] = 0;
	}	

	// butterworth filter default values
	*attr_NumSamples_read = 16384;
	if (plane == "Horizontal") {
		*attr_TuneMin_read = 0.295-0.05;
		*attr_TuneMax_read = 0.295+0.05;
	}
	else if (plane == "Vertical") {
		*attr_TuneMin_read = 0.185-0.05;
		*attr_TuneMax_read = 0.185+0.05;
	}

	*attr_DebugMinThres_read = 0;
	*attr_DebugMaxThres_read = 0.5;

	*attr_TuneNotch_read = 0.05;
	*attr_TuneNotchWidth_read = 0.002;	

	*attr_TuneMinPeakThresh_read = 1;
	*attr_MaxNumBpmUsed_read = 2;
	*attr_LPCoeff_read = 0.01;

	*attr_InhibitCycles_read = 20;

	*attr_InhibitSegmentSize_read = 1000;

	*attr_HistoryBufSize_read = 10000;	

	*attr_EnableDespiking_read = false;
	*attr_DespikingFilterLen_read = 3;
	*attr_Frequency_read = 0;	
	*attr_FrequencySet_read = 1000;

	*attr_InjPeakDetectThresh_read = 0.5; // mm
	*attr_SumThres_read = 1500000;
	*attr_TunePeakAmplitude_read = 0;

	*attr_EnableInjDespiking_read = false;

	tune_buf_idx = 0;
	debug_buf_idx = 0;
	skip_cnt = *attr_InhibitCycles_read;

}


/*
 * function which split string (once defined a separator 
 */
void TuneGof2::string_explode(string str, string separator, vector<string>* results)
{
	size_t found;

	found = str.find_first_of(separator);
	while(found != string::npos) {
		if(found > 0) {
			results->push_back(str.substr(0,found));
		}
		str = str.substr(found+1);
		found = str.find_first_of(separator);
	}
	if(str.length() > 0) {
		results->push_back(str);
	}

}


void TuneGof2::init_fft(char *fft_wisdom)	
{

	fft_mutex->lock();

	gof2_fft_out = (fftwf_complex *)fftwf_malloc(sizeof(fftwf_complex) * GOF2_BUF_SIZE);
	
	fft_mutex->unlock();
		
} /* end init_fft() */	



void TuneGof2::deinit_fft()
{
	fft_mutex->lock();
	fftwf_free(gof2_fft_out);
	fft_mutex->unlock();
		
} /* deinit_fft() */


int32_t TuneGof2::get_bpm_index_from_label(const char *input_string)
{
	int bpm_indexes[2];
	int num_indexes;

	extract_numbers(input_string, bpm_indexes, &num_indexes, 2);
	
	for (int i = 0; i < GOF2_NUM_BPM_ACTIVED; i++) {
		if ((bpm_indexes[0] == bpm_addr_tb[i][0]) && (bpm_indexes[1] == bpm_addr_tb[i][1]))
			return bpm_addr_tb[i][4];
	}

	Tango::Except::throw_exception(
		(const char *)"Failed to get bpm index",
		(const char *)"Bpm name not found",
		__FUNCTION__);

}


void TuneGof2::extract_numbers(const char *str, int numbers[], int *count, int max_count) 
{
	*count = 0; 
	const char *ptr = str;

	while (*ptr) {
		if (isdigit(*ptr)) {
			numbers[*count] = strtol(ptr, (char **)&ptr, 10);
			(*count)++; 
			if ((*count) == max_count) // expect only two numbers in the bpm label; ex: bpm_s1.2
				break;
		} else {
			ptr++; 
		}
	}
}


void TuneGof2::get_bpm_fftdata(Tango::DevVarULongArray *argin, int *int_peak_found, int *ext_peak_found, int *sum_err)
{

	//	Add your own code
	if (argin->length() < 4) {
		Tango::Except::throw_exception(
			(const char *) "Failed to get bpm values",
			(const char *) "Invalid number of input parameters (num_bpm_num_samples,data_type(0,1,..X(6),Y(7))",
			__FUNCTION__, Tango::ERR);
	}

	struct timeval start, end;

	gettimeofday(&start,NULL);

	int bpm_off = 2, type_off = 2;	

	int64_t num_samples = (*argin)[0], num_type  = (*argin)[1];
	 
	int32_t num_bpm = argin->length()-2-num_type; 
	bpm_off = 2 + num_type;

	int32_t tot_type = 0;

	if ((num_samples < 0) || (num_samples > GOF2_BUF_SIZE)) {
		Tango::Except::throw_exception(
			(const char *) "Failed to get bpm values",
			(const char *) "Invalid number of requested samples",
			__FUNCTION__, Tango::ERR);
	}

	for (int i = 0; i <num_type; i++) {
		tot_type += 1;
		if (((*argin)[i+type_off] < 0) || ((*argin)[i+type_off] > (GOF2_NUM_DATA_TYPES-4))) {
			Tango::Except::throw_exception(
				(const char *) "Failed to get bpm values",
				(const char *) "Invalid requested data type (0..7)",
				__FUNCTION__, Tango::ERR);
		}
	}

	for (int i = 0; i <num_bpm; i++) {
		if (((*argin)[i+bpm_off] < 0) || ((*argin)[i+bpm_off] >= GOF2_NUM_BPM_ACTIVED)) {
			Tango::Except::throw_exception(
				(const char *) "Failed to get bpm values",
				(const char *) "Invalid bpm index",
				__FUNCTION__, Tango::ERR);
		}
	}

	int32_t ret_data = num_samples;

	// return the full buffer values

	int32_t start1[GOF2_NUM_BPM_ACTIVED], end1[GOF2_NUM_BPM_ACTIVED], 
		start2[GOF2_NUM_BPM_ACTIVED], end2[GOF2_NUM_BPM_ACTIVED], 
		cnt = 0;
	int32_t bpm_idx[GOF2_NUM_BPM_ACTIVED], buf_idx[GOF2_NUM_BPM_ACTIVED], buf_cnt[GOF2_NUM_BPM_ACTIVED];

	// get stored data bpm index from sr bpm index
	for (int i = 0; i < num_bpm; i++) {
		bpm_idx[i] = (*argin)[i+bpm_off];
		
	}

	// get buffer indexes	
	for (int i = 0; i < num_bpm; i++) {
		buf_cnt[i] = gof2->bpm[bpm_idx[i]].buf_cnt;
		buf_idx[i] = gof2->bpm[bpm_idx[i]].buf_idx; // buffer idx
	}	

	// calculate start/stop indexes
	for (int i = 0; i < num_bpm; i++) {
	
		int32_t tot_buf_size;
		if (buf_cnt[i] >= GOF2_BUF_SIZE)
			buf_cnt[i] = GOF2_BUF_SIZE;
		else
			buf_cnt[i] = GOF2_BUF_SIZE;

		if ((buf_idx[i] - num_samples) >= 0) {
			start1[i] = buf_idx[i] - num_samples;
			end1[i] = start1[i] + num_samples;
			start2[i] = 0; end2[i] = 0;
		}
		else {
			start1[i] = buf_cnt[i] - (num_samples - buf_idx[i] + 1);
			end1[i] = buf_cnt[i];
			start2[i] = 0;
			end2[i] = buf_idx[i];
		}
	}

	// ************* REAL DATA *********** 
	// 0 uint32_t Va          ; // amplitude A
	// 1 uint32_t Vb          ; // amplitude B
	// 2 uint32_t Vc          ; // amplitude C
	// 3 uint32_t Vd          ; // amplitude D
	// 4 uint32_t Sum         ; // amplitudes' sum
	// 5 int32_t Q            ; // q offset
	// 6 int32_t X            ; // x offset     current hor
	// 7 int32_t Y            ; // y offset     current ver
	// ************* PILOT TONE ***********
	// 8 uint32_t Va          ; // amplitude A
	// 9 uint32_t Vb          ; // amplitude B
	// 10 uint32_t Vc          ; // amplitude C
	// 11 uint32_t Vd          ; // amplitude D
	// 12 uint32_t Sum         ; // amplitudes' sum
	// 13 int32_t Q            ; // q offset
	// 14 int32_t X            ; // x offset
	// 15 int32_t Y            ; // y offset

	double value_start[GOF2_NUM_BPM_ACTIVED],value_middle[GOF2_NUM_BPM_ACTIVED], value_end[GOF2_NUM_BPM_ACTIVED];
	int samples_cnt = 0;

	for (int k = 0; k < num_type; k++) {
		int data_type = (*argin)[type_off+k];
		for (int j = 0; j < num_bpm; j++) {
			cnt = 0;

			if (data_type <= 7) {
				for (int i = start1[j]; i < end1[j]; i++) {
					gof2_data_in[j][cnt++] = *(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].Va+data_type);
					debug_circ_buf[debug_buf_idx] = *(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].Va+data_type);
					if (++debug_buf_idx >= TUNEGOF2_DEBUGBUF_SIZE)
						debug_buf_idx = 0;
				}
				for (int i = start2[j]; i < end2[j]; i++) {
					gof2_data_in[j][cnt++] = *(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].Va+data_type);
					debug_circ_buf[debug_buf_idx] = *(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].Va+data_type);
					if (++debug_buf_idx >= TUNEGOF2_DEBUGBUF_SIZE)
						debug_buf_idx = 0;
				}
			}
		} // num_bpm
	}


	// calcolo il valore medio, il ringing dell'iniezione da contributo DC nullo, monitoro solo il canale orizzontale
	// se il massimo sta tra 1000 ed N-1000 allora viene notificatop che e' stato trovato il picco dell'iniezione
	// e il picco della FFT viene salvato oltre che nel buffer circolare anche nell'attributo InjTune
	// Si verifica che il segnale somma sia sopra soglia in modo da invalidare o meno la misura
	for (int j = 0; j < num_bpm; j++) {
		int64_t avg_X = 0;
		uint32_t min_Sum = 1e9;
		// calculate average 
		for (int i = start1[j]; i < end1[j]; i++) {
			avg_X += *(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].X);
			if (*(int32_t *)(&gof2->bpm[bpm_idx[j]].bpm_data[i].Sum) < min_Sum)
				min_Sum = *(int32_t *)(&gof2->bpm[bpm_idx[j]].bpm_data[i].Sum);
		}
		for (int i = start2[j]; i < end2[j]; i++) {
			avg_X += *(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].X);
			if (*(int32_t *)(&gof2->bpm[bpm_idx[j]].bpm_data[i].Sum) < min_Sum)
				min_Sum = *(int32_t *)(&gof2->bpm[bpm_idx[j]].bpm_data[i].Sum);
		}
		avg_X /= num_samples;

		

		// find the horizontal peak of the injection inside the interval [*attr_InhibitSegmentSize_read num_samples-*attr_InhibitSegmentSize_read]
		// check the Sum threshold to invalidate the measurement
		int cnt = 0;
		int_peak_found[j] = 0;ext_peak_found[j] = 0;sum_err[j] = 0;
		for (int i = start1[j]; i < end1[j]; i++) {
			if (abs(*(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].X)-(int32_t)avg_X) > (int32_t)(*attr_InjPeakDetectThresh_read * 1000000)) {
				if ((cnt > *attr_InhibitSegmentSize_read) && (cnt < num_samples - *attr_InhibitSegmentSize_read)) {
					int_peak_found[j]++;
				}
				if ((cnt < *attr_InhibitSegmentSize_read) || (cnt > num_samples - *attr_InhibitSegmentSize_read)) {
					ext_peak_found[j]++;
				}
			}
			cnt++;
		}
		for (int i = start2[j]; i < end2[j]; i++) {
			if (abs(*(int32_t *) (&gof2->bpm[bpm_idx[j]].bpm_data[i].X)-(int32_t)avg_X) > (int32_t)(*attr_InjPeakDetectThresh_read * 1000000)) {
				if ((cnt > *attr_InhibitSegmentSize_read) && (cnt < num_samples - *attr_InhibitSegmentSize_read)) {
					int_peak_found[j]++;
				}
				if ((cnt < *attr_InhibitSegmentSize_read) || (cnt > num_samples - *attr_InhibitSegmentSize_read)) {
					ext_peak_found[j]++;
				}
			}
			cnt++;
		}

		if (min_Sum < *attr_InjPeakDetectThresh_read)
			sum_err[j]++;

		if (min_Sum < *attr_SumThres_read)
			sum_err[j] += 100;

	}


	if (*attr_EnableInjDespiking_read) {
		for (int i = 0; i < num_bpm; i++) {
			if (ext_peak_found[i]) {			
				skip_cnt = *attr_InhibitCycles_read - 1;
				break;
			}
		}
	}

	// block FFT update not just for a cycle but for *attr_InhibitCycles_read cycles
	if (skip_cnt < *attr_InhibitCycles_read) {
		skip_cnt--;
		if (skip_cnt == 0)
			skip_cnt = *attr_InhibitCycles_read;
		return;
	}

	int notch_start_idx = *attr_TuneNotch_read * num_samples - *attr_TuneNotchWidth_read * num_samples;
	int notch_end_idx = *attr_TuneNotch_read * num_samples + *attr_TuneNotchWidth_read * num_samples;	

	// perform FFT	
	for (int k = 0; k < num_type; k++) {
		for (int j = 0; j < num_bpm; j++) {

			fftwf_plan plan = fftwf_plan_dft_r2c_1d(num_samples, gof2_data_in[j], gof2_fft_out, FFTW_ESTIMATE);
			fftwf_execute(plan);

			tune_fft_bpm_data[j][0] = 0;
			for (size_t t = 1; t < num_samples/2; t++) {

				if ((t <= notch_start_idx) || (t >= notch_end_idx)) {
					tune_fft_bpm_data[j][t] = sqrt(gof2_fft_out[t][0]*gof2_fft_out[t][0] + 
						gof2_fft_out[t][1]*gof2_fft_out[t][1])/(num_samples/2);
				}
				else {
					tune_fft_bpm_data[j][t] = 0;
				}

			}
			fftwf_destroy_plan(plan);
		}
	}

}


float TuneGof2::median(std::vector<float>& array) 
{
	// Create a copy of the input vector
	std::vector<float> copy = array;

	// Sort the copy using std::sort
	std::sort(copy.begin(), copy.end());

	float median;
	size_t length = copy.size();
	if (length % 2 == 0) {
		// If the length is even, median is the average of the two middle values
		median = (copy[length / 2 - 1] + copy[length / 2]) / 2.0;
	} else {
		// If the length is odd, median is the middle value
		median = copy[length / 2];
	}

    return median;
}


struct timespec TuneGof2::add_timespec(struct timespec ts1, struct timespec ts2) 
{
	struct timespec result;

	result.tv_sec = ts1.tv_sec + ts2.tv_sec; // Somma i secondi

	result.tv_nsec = ts1.tv_nsec + ts2.tv_nsec; // Somma i nanosecondi

	// Gestisci il riporto se i nanosecondi superano 1 miliardo
	if (result.tv_nsec >= 1000000000) {
		result.tv_sec += result.tv_nsec / 1000000000; // Incrementa i secondi
		result.tv_nsec = result.tv_nsec % 1000000000; // Rimuovi il miliardo dai nanosecondi
	}

	return result;
}


struct timespec TuneGof2::subtract_timespec(struct timespec ts1, struct timespec ts2) 
{
    struct timespec result;

    // Sottrai i secondi e i nanosecondi
    result.tv_sec = ts1.tv_sec - ts2.tv_sec;
    result.tv_nsec = ts1.tv_nsec - ts2.tv_nsec;

    // Se i nanosecondi sono negativi, fai il riporto
    if (result.tv_nsec < 0) {
        result.tv_sec -= 1;                  // Decrementa i secondi
        result.tv_nsec += 1000000000;        // Aggiungi un miliardo ai nanosecondi
    }

    return result;
}


long TuneGof2::timespec_to_nsec(struct timespec ts) 
{
    return (ts.tv_sec * 1000000000L) + ts.tv_nsec;
}


/* clang-format off */// //--------------------------------------------------------



// //--------------------------------------------------------
// /**
//  *	Read attribute DetectMinPeak related method
//  *
//  *
//  *	Data type:	Tango::DevBoolean
//  *	Attr type:	Scalar
//  */
// //--------------------------------------------------------
// void TuneGof2::read_DetectMinPeak(Tango::Attribute &attr)
// {
// 	DEBUG_STREAM << "TuneGof2::read_DetectMinPeak(Tango::Attribute &attr) entering... " << std::endl;
// 	/* clang-format on */
// 	//	Set the attribute value
// 	attr.set_value(attr_DetectMinPeak_read);
// 	/* clang-format off */
// }

// //--------------------------------------------------------
// /**
//  *	Write attribute DetectMinPeak related method
//  *
//  *
//  *	Data type:	Tango::DevBoolean
//  *	Attr type:	Scalar
//  */
// //--------------------------------------------------------
// void TuneGof2::write_DetectMinPeak(Tango::WAttribute &attr)
// {
// 	DEBUG_STREAM << "TuneGof2::write_DetectMinPeak(Tango::WAttribute &attr) entering... " << std::endl;
// 	//	Retrieve write value
// 	Tango::DevBoolean	w_val;
// 	attr.get_write_value(w_val);
// 	/* clang-format on */
// 	//	Add your own code
// 	*attr_DetectMinPeak_read = w_val;
// 	/* clang-format off */
// }


/*----- PROTECTED REGION END -----*/	//	TuneGof2::namespace_ending
} //	namespace
