/*----- PROTECTED REGION ID(TuneGof2.h) ENABLED START -----*/
/* clang-format on */
//=============================================================================
//
// file :        TuneGof2.h
//
// description : Include file for the TuneGof2 class
//
// project :     GOF2 Tune measurement
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef TuneGof2_H
#define TuneGof2_H

#define TUNEGOF2_MAX_NUM_BPM 8
#define TUNEGOF2_NUM_DATA_TYPES 2
#define TUNEGOF2_LPF_RESPONSE_TIME 500 // max low pass filter response time in samples

#include <tango/tango.h>

#include <signal.h>
#include <fftw3.h>
#include <itpp/itbase.h>
#include <itpp/itcomm.h>

#include <gof2_shm.h>
#include <gof2_fun.h>

#define TUNEGOF2_CIRCBUF_SIZE 2000000
#define TUNEGOF2_DEBUGBUF_SIZE 200000
#define TUNEGOF2_MAX_SKIP_CNT 20
#define TUNEOGF2_RESCALE 10000


class tunethread;

/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2.h

#ifdef TANGO_LOG
	// cppTango after c934adea (Merge branch 'remove-cout-definition' into 'main', 2022-05-23)
	// nothing to do
#else
	// cppTango 9.3-backports and older
	#define TANGO_LOG       cout
	#define TANGO_LOG_INFO  cout2
	#define TANGO_LOG_DEBUG cout3
#endif // TANGO_LOG

/**
 *  TuneGof2 class description:
 *    
 */


namespace TuneGof2_ns
{
/*----- PROTECTED REGION ID(TuneGof2::Additional Class Declarations) ENABLED START -----*/
/* clang-format on */
//	Additional Class Declarations
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2::Additional Class Declarations

class TuneGof2 : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(TuneGof2::Data Members) ENABLED START -----*/
/* clang-format on */
//	Add your own data members

public:
	friend class tunethread;

	int fd_shm;
	void *shm_addr;

	struct gof2_shm_t *gof2;

	FILE *tune_wis;  
	omni_mutex *fft_mutex;    
	fftwf_complex *gof2_fft_out;
	fftwf_plan gof2_fft_plan;
	
	float gof2_data_in[TUNEGOF2_MAX_NUM_BPM][GOF2_BUF_SIZE];

	// fft data of all bpms used for tune calculation
	double tune_fft_bpm_data[TUNEGOF2_MAX_NUM_BPM][GOF2_BUF_SIZE];

	// product of the ffts of the bpms selected 
	double tune_fft_raw_data[GOF2_BUF_SIZE];
	double tune_fft_filt_data[GOF2_BUF_SIZE];

	// buffer used for filtfit implementation
	double filter_compensation_vector[GOF2_BUF_SIZE],
		filtbuf_tmp[GOF2_BUF_SIZE];

	tunethread *tuneloop;

	bool abort_tune_flag, run_tune_flag, err_tune_flag, enable_hugepages;

	float tune_circ_buf[TUNEGOF2_CIRCBUF_SIZE], debug_circ_buf[TUNEGOF2_DEBUGBUF_SIZE];
	int tune_buf_idx;
	int debug_buf_idx;
	int skip_cnt;



/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2::Data Members

//	Device property data members
public:
	//	HugePages:	
	Tango::DevBoolean	hugePages;
	//	SharedMemoryID:	
	std::string	sharedMemoryID;
	//	BpmIndexList:	
	std::vector<Tango::DevLong>	bpmIndexList;
	//	Plane:	
	std::string	plane;
	//	AutoStart:	
	Tango::DevBoolean	autoStart;

//	Attribute data members
public:
	Tango::DevLong	*attr_NumSamples_read;
	Tango::DevFloat	*attr_TuneMin_read;
	Tango::DevFloat	*attr_TuneMax_read;
	Tango::DevFloat	*attr_TuneMinPeakThresh_read;
	Tango::DevFloat	*attr_Tune_read;
	Tango::DevLong	*attr_MaxNumBpmUsed_read;
	Tango::DevDouble	*attr_LPCoeff_read;
	Tango::DevDouble	*attr_TuneNotch_read;
	Tango::DevDouble	*attr_TuneNotchWidth_read;
	Tango::DevLong	*attr_InhibitCycles_read;
	Tango::DevLong	*attr_InhibitSegmentSize_read;
	Tango::DevDouble	*attr_DebugMinThres_read;
	Tango::DevDouble	*attr_DebugMaxThres_read;
	Tango::DevLong	*attr_HistoryBufSize_read;
	Tango::DevBoolean	*attr_EnableDespiking_read;
	Tango::DevLong	*attr_DespikingFilterLen_read;
	Tango::DevDouble	*attr_Frequency_read;
	Tango::DevDouble	*attr_FrequencySet_read;
	Tango::DevFloat	*attr_InjTune_read;
	Tango::DevDouble	*attr_InjPeakDetectThresh_read;
	Tango::DevULong	*attr_SumThres_read;
	Tango::DevBoolean	*attr_EnableInjDespiking_read;
	Tango::DevDouble	*attr_TunePeakAmplitude_read;
	Tango::DevDouble	*attr_FFTRaw_read;
	Tango::DevDouble	*attr_FFTFilt_read;
	Tango::DevDouble	*attr_BpmIndexList_read;
	Tango::DevFloat	*attr_TuneHistory_read;
	Tango::DevFloat	*attr_Debug_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	TuneGof2(Tango::DeviceClass *cl,std::string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	TuneGof2(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	TuneGof2(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~TuneGof2();


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : TuneGof2::read_attr_hardware()
	 * Description:  Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(std::vector<long> &attr_list);
	//--------------------------------------------------------
	/*
	 *	Method      : TuneGof2::write_attr_hardware()
	 * Description:  Hardware writing for attributes.
	 */
	//--------------------------------------------------------
	virtual void write_attr_hardware(std::vector<long> &attr_list);

/**
 *	Attribute NumSamples related methods
 *
 *
 *	Data type:  Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_NumSamples(Tango::Attribute &attr);
	virtual void write_NumSamples(Tango::WAttribute &attr);
	virtual bool is_NumSamples_allowed(Tango::AttReqType type);
/**
 *	Attribute TuneMin related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Scalar
 */
	virtual void read_TuneMin(Tango::Attribute &attr);
	virtual void write_TuneMin(Tango::WAttribute &attr);
	virtual bool is_TuneMin_allowed(Tango::AttReqType type);
/**
 *	Attribute TuneMax related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Scalar
 */
	virtual void read_TuneMax(Tango::Attribute &attr);
	virtual void write_TuneMax(Tango::WAttribute &attr);
	virtual bool is_TuneMax_allowed(Tango::AttReqType type);
/**
 *	Attribute TuneMinPeakThresh related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Scalar
 */
	virtual void read_TuneMinPeakThresh(Tango::Attribute &attr);
	virtual void write_TuneMinPeakThresh(Tango::WAttribute &attr);
	virtual bool is_TuneMinPeakThresh_allowed(Tango::AttReqType type);
/**
 *	Attribute Tune related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Scalar
 */
	virtual void read_Tune(Tango::Attribute &attr);
	virtual bool is_Tune_allowed(Tango::AttReqType type);
/**
 *	Attribute MaxNumBpmUsed related methods
 * Description:  Number of bpms used for tune estimation
 *
 *	Data type:  Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_MaxNumBpmUsed(Tango::Attribute &attr);
	virtual void write_MaxNumBpmUsed(Tango::WAttribute &attr);
	virtual bool is_MaxNumBpmUsed_allowed(Tango::AttReqType type);
/**
 *	Attribute LPCoeff related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_LPCoeff(Tango::Attribute &attr);
	virtual void write_LPCoeff(Tango::WAttribute &attr);
	virtual bool is_LPCoeff_allowed(Tango::AttReqType type);
/**
 *	Attribute TuneNotch related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_TuneNotch(Tango::Attribute &attr);
	virtual void write_TuneNotch(Tango::WAttribute &attr);
	virtual bool is_TuneNotch_allowed(Tango::AttReqType type);
/**
 *	Attribute TuneNotchWidth related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_TuneNotchWidth(Tango::Attribute &attr);
	virtual void write_TuneNotchWidth(Tango::WAttribute &attr);
	virtual bool is_TuneNotchWidth_allowed(Tango::AttReqType type);
/**
 *	Attribute InhibitCycles related methods
 *
 *
 *	Data type:  Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_InhibitCycles(Tango::Attribute &attr);
	virtual void write_InhibitCycles(Tango::WAttribute &attr);
	virtual bool is_InhibitCycles_allowed(Tango::AttReqType type);
/**
 *	Attribute InhibitSegmentSize related methods
 *
 *
 *	Data type:  Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_InhibitSegmentSize(Tango::Attribute &attr);
	virtual void write_InhibitSegmentSize(Tango::WAttribute &attr);
	virtual bool is_InhibitSegmentSize_allowed(Tango::AttReqType type);
/**
 *	Attribute DebugMinThres related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_DebugMinThres(Tango::Attribute &attr);
	virtual void write_DebugMinThres(Tango::WAttribute &attr);
	virtual bool is_DebugMinThres_allowed(Tango::AttReqType type);
/**
 *	Attribute DebugMaxThres related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_DebugMaxThres(Tango::Attribute &attr);
	virtual void write_DebugMaxThres(Tango::WAttribute &attr);
	virtual bool is_DebugMaxThres_allowed(Tango::AttReqType type);
/**
 *	Attribute HistoryBufSize related methods
 *
 *
 *	Data type:  Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_HistoryBufSize(Tango::Attribute &attr);
	virtual void write_HistoryBufSize(Tango::WAttribute &attr);
	virtual bool is_HistoryBufSize_allowed(Tango::AttReqType type);
/**
 *	Attribute EnableDespiking related methods
 *
 *
 *	Data type:  Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_EnableDespiking(Tango::Attribute &attr);
	virtual void write_EnableDespiking(Tango::WAttribute &attr);
	virtual bool is_EnableDespiking_allowed(Tango::AttReqType type);
/**
 *	Attribute DespikingFilterLen related methods
 *
 *
 *	Data type:  Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_DespikingFilterLen(Tango::Attribute &attr);
	virtual void write_DespikingFilterLen(Tango::WAttribute &attr);
	virtual bool is_DespikingFilterLen_allowed(Tango::AttReqType type);
/**
 *	Attribute Frequency related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Frequency(Tango::Attribute &attr);
	virtual bool is_Frequency_allowed(Tango::AttReqType type);
/**
 *	Attribute FrequencySet related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_FrequencySet(Tango::Attribute &attr);
	virtual void write_FrequencySet(Tango::WAttribute &attr);
	virtual bool is_FrequencySet_allowed(Tango::AttReqType type);
/**
 *	Attribute InjTune related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Scalar
 */
	virtual void read_InjTune(Tango::Attribute &attr);
	virtual bool is_InjTune_allowed(Tango::AttReqType type);
/**
 *	Attribute InjPeakDetectThresh related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_InjPeakDetectThresh(Tango::Attribute &attr);
	virtual void write_InjPeakDetectThresh(Tango::WAttribute &attr);
	virtual bool is_InjPeakDetectThresh_allowed(Tango::AttReqType type);
/**
 *	Attribute SumThres related methods
 *
 *
 *	Data type:  Tango::DevULong
 *	Attr type:	Scalar
 */
	virtual void read_SumThres(Tango::Attribute &attr);
	virtual void write_SumThres(Tango::WAttribute &attr);
	virtual bool is_SumThres_allowed(Tango::AttReqType type);
/**
 *	Attribute EnableInjDespiking related methods
 *
 *
 *	Data type:  Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_EnableInjDespiking(Tango::Attribute &attr);
	virtual void write_EnableInjDespiking(Tango::WAttribute &attr);
	virtual bool is_EnableInjDespiking_allowed(Tango::AttReqType type);
/**
 *	Attribute TunePeakAmplitude related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_TunePeakAmplitude(Tango::Attribute &attr);
	virtual bool is_TunePeakAmplitude_allowed(Tango::AttReqType type);
/**
 *	Attribute FFTRaw related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Spectrum max = 4000000
 */
	virtual void read_FFTRaw(Tango::Attribute &attr);
	virtual bool is_FFTRaw_allowed(Tango::AttReqType type);
/**
 *	Attribute FFTFilt related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Spectrum max = 4000000
 */
	virtual void read_FFTFilt(Tango::Attribute &attr);
	virtual bool is_FFTFilt_allowed(Tango::AttReqType type);
/**
 *	Attribute BpmIndexList related methods
 *
 *
 *	Data type:  Tango::DevDouble
 *	Attr type:	Spectrum max = 299
 */
	virtual void read_BpmIndexList(Tango::Attribute &attr);
	virtual void write_BpmIndexList(Tango::WAttribute &attr);
	virtual bool is_BpmIndexList_allowed(Tango::AttReqType type);
/**
 *	Attribute TuneHistory related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Spectrum max = 1000000
 */
	virtual void read_TuneHistory(Tango::Attribute &attr);
	virtual bool is_TuneHistory_allowed(Tango::AttReqType type);
/**
 *	Attribute Debug related methods
 *
 *
 *	Data type:  Tango::DevFloat
 *	Attr type:	Spectrum max = 2000000
 */
	virtual void read_Debug(Tango::Attribute &attr);
	virtual bool is_Debug_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : TuneGof2::add_dynamic_attributes()
	 * Description:  Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();




//	Command related methods
public:
	/**
	 *	Command Start related method
	 *
	 *
	 */
	virtual void start();
	virtual bool is_Start_allowed(const CORBA::Any &any);
	/**
	 *	Command Stop related method
	 *
	 *
	 */
	virtual void stop();
	virtual bool is_Stop_allowed(const CORBA::Any &any);
	/**
	 *	Command GetTune related method
	 *
	 *
	 *	@param argin mode(0);num samples
	 *	@returns
	 */
	virtual Tango::DevVarFloatArray *get_tune(const Tango::DevVarLongArray *argin);
	virtual bool is_GetTune_allowed(const CORBA::Any &any);


	//--------------------------------------------------------
	/**
	 *	Method      : TuneGof2::add_dynamic_commands()
	 * Description:  Add dynamic commands if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_commands();

/*----- PROTECTED REGION ID(TuneGof2::Additional Method prototypes) ENABLED START -----*/
/* clang-format on */
//	Additional Method prototypes

	void allocate_shm(void);
	void extract_numbers(const char *str, int numbers[], int *count, int max_count);
	void string_explode(string str, string separator, vector<string>* results);
	void init_fft(char *fft_wisdom);
	void deinit_fft(void);
	void modify_property(string property_name, string separator, string bpm_name, string elem_name, double new_value);
	int32_t get_bpm_index_from_label(const char *input_string);
	void filtfilt(double *datain, double *dataout, double *datatmp, int num_samples);
	void calc_compensation(int num_samples);
	void init_data(void);	
	void get_bpm_fftdata(Tango::DevVarULongArray *argin, int *int_peak_found, int *ext_peak_found, int *sum_err);
	float median(std::vector<float>& array);
	struct timespec add_timespec(struct timespec ts1, struct timespec ts2);
	struct timespec subtract_timespec(struct timespec ts1, struct timespec ts2);
	long timespec_to_nsec(struct timespec ts);

/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2::Additional Method prototypes
};

/*----- PROTECTED REGION ID(TuneGof2::Additional Classes Definitions) ENABLED START -----*/
/* clang-format on */
//	Additional Classes Definitions
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2::Additional Classes Definitions

}	//	End of namespace

#endif   //	TuneGof2_H
