/*----- PROTECTED REGION ID(TuneGof2Class.cpp) ENABLED START -----*/
/* clang-format on */
//=============================================================================
//
// file :        TuneGof2Class.cpp
//
// description : C++ source for the TuneGof2Class.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the TuneGof2 once per process.
//
// project :     GOF2 Tune measurement
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include "TuneGof2Class.h"
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2Class.cpp

//-------------------------------------------------------------------
/**
 *	Create TuneGof2Class singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_TuneGof2_class(const char *name) {
		return TuneGof2_ns::TuneGof2Class::init(name);
	}
}

namespace TuneGof2_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
TuneGof2Class *TuneGof2Class::_instance = NULL;

//===================================================================
//	Class constants
//===================================================================
constexpr long FFTRawAttrib::X_DATA_SIZE;
constexpr long FFTFiltAttrib::X_DATA_SIZE;
constexpr long BpmIndexListAttrib::X_DATA_SIZE;
constexpr long TuneHistoryAttrib::X_DATA_SIZE;
constexpr long DebugAttrib::X_DATA_SIZE;
//--------------------------------------------------------
/**
 * method : 		TuneGof2Class::TuneGof2Class(std::string &s)
 * description : 	constructor for the TuneGof2Class
 *
 * @param s	The class name
 */
//--------------------------------------------------------
TuneGof2Class::TuneGof2Class(std::string &s):Tango::DeviceClass(s)
{
	TANGO_LOG_INFO << "Entering TuneGof2Class constructor" << std::endl;
	set_default_property();
	write_class_property();

	/*----- PROTECTED REGION ID(TuneGof2Class::constructor) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::constructor

	TANGO_LOG_INFO << "Leaving TuneGof2Class constructor" << std::endl;
}

//--------------------------------------------------------
/**
 * method : 		TuneGof2Class::~TuneGof2Class()
 * description : 	destructor for the TuneGof2Class
 */
//--------------------------------------------------------
TuneGof2Class::~TuneGof2Class()
{
	/*----- PROTECTED REGION ID(TuneGof2Class::destructor) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		TuneGof2Class::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
TuneGof2Class *TuneGof2Class::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			std::string s(name);
			_instance = new TuneGof2Class(s);
		}
		catch (std::bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		TuneGof2Class::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
TuneGof2Class *TuneGof2Class::instance()
{
	if (_instance == NULL)
	{
		std::cerr << "Class is not initialised !!" << std::endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================
//--------------------------------------------------------
/**
 * method : 		StartClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *StartClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	TANGO_LOG_INFO << "StartClass::execute(): arrived" << std::endl;
	((static_cast<TuneGof2 *>(device))->start());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		StopClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *StopClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	TANGO_LOG_INFO << "StopClass::execute(): arrived" << std::endl;
	((static_cast<TuneGof2 *>(device))->stop());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		GetTuneClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *GetTuneClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	TANGO_LOG_INFO << "GetTuneClass::execute(): arrived" << std::endl;
	const Tango::DevVarLongArray *argin;
	extract(in_any, argin);
	return insert((static_cast<TuneGof2 *>(device))->get_tune(argin));
}


//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::get_class_property()
 * Description:  Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum TuneGof2Class::get_class_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::get_default_device_property()
 * Description:  Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum TuneGof2Class::get_default_device_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::get_default_class_property()
 * Description:  Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum TuneGof2Class::get_default_class_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}


//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::set_default_property()
 * Description:  Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void TuneGof2Class::set_default_property()
{
	std::string	prop_name;
	std::string	prop_desc;
	std::string	prop_def;
	std::vector<std::string>	vect_data;

	//	Set Default Class Properties

	//	Set Default device Properties
	prop_name = "HugePages";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "SharedMemoryID";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "BpmIndexList";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Plane";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "AutoStart";
	prop_desc = "";
	prop_def  = "true";
	vect_data.clear();
	vect_data.push_back("true");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::write_class_property()
 * Description:  Set class description fields as property in database
 */
//--------------------------------------------------------
void TuneGof2Class::write_class_property()
{
//	First time, check if database used
if (Tango::Util::_UseDb == false)
	return;

Tango::DbData	data;
std::string	classname = get_name();
std::string	header;

//	Put title
Tango::DbDatum	title("ProjectTitle");
std::string	str_title("GOF2 Tune measurement");
title << str_title;
data.push_back(title);

//	Put Description
Tango::DbDatum	description("Description");
std::vector<std::string>	str_desc;
str_desc.push_back("");
description << str_desc;
data.push_back(description);

//  Put inheritance
Tango::DbDatum	inher_datum("InheritedFrom");
std::vector<std::string> inheritance;
inheritance.push_back("TANGO_BASE_CLASS");
inher_datum << inheritance;
data.push_back(inher_datum);

//	Call database and and values
get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::device_factory()
 * Description:  Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void TuneGof2Class::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
/*----- PROTECTED REGION ID(TuneGof2Class::device_factory_before) ENABLED START -----*/
/* clang-format on */
//	Add your own code
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::device_factory_before

//	Create devices and add it into the device list
for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
{
	TANGO_LOG_DEBUG << "Device name : " << (*devlist_ptr)[i].in() << std::endl;
	device_list.push_back(new TuneGof2(this, (*devlist_ptr)[i]));
}

//	Manage dynamic attributes if any
erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

//	Export devices to the outside world
for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
{
	//	Add dynamic attributes if any
	TuneGof2 *dev = static_cast<TuneGof2 *>(device_list[device_list.size()-i]);
	dev->add_dynamic_attributes();

	//	Check before if database used.
	if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
		export_device(dev);
	else
		export_device(dev, dev->get_name().c_str());
}

/*----- PROTECTED REGION ID(TuneGof2Class::device_factory_after) ENABLED START -----*/
/* clang-format on */
//	Add your own code
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::attribute_factory()
 * Description:  Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void TuneGof2Class::attribute_factory(std::vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(TuneGof2Class::attribute_factory_before) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::attribute_factory_before
	//	Attribute : NumSamples
	NumSamplesAttrib	*numsamples = new NumSamplesAttrib();
	Tango::UserDefaultAttrProp	numsamples_prop;
	//	description	not set for NumSamples
	//	label	not set for NumSamples
	//	unit	not set for NumSamples
	//	standard_unit	not set for NumSamples
	//	display_unit	not set for NumSamples
	//	format	not set for NumSamples
	//	max_value	not set for NumSamples
	//	min_value	not set for NumSamples
	//	max_alarm	not set for NumSamples
	//	min_alarm	not set for NumSamples
	//	max_warning	not set for NumSamples
	//	min_warning	not set for NumSamples
	//	delta_t	not set for NumSamples
	//	delta_val	not set for NumSamples
	numsamples->set_default_properties(numsamples_prop);
	//	Not Polled
	numsamples->set_disp_level(Tango::OPERATOR);
	numsamples->set_memorized();
	numsamples->set_memorized_init(true);
	att_list.push_back(numsamples);

	//	Attribute : TuneMin
	TuneMinAttrib	*tunemin = new TuneMinAttrib();
	Tango::UserDefaultAttrProp	tunemin_prop;
	//	description	not set for TuneMin
	//	label	not set for TuneMin
	//	unit	not set for TuneMin
	//	standard_unit	not set for TuneMin
	//	display_unit	not set for TuneMin
	tunemin_prop.set_format("%.4f");
	tunemin_prop.set_max_value("1");
	tunemin_prop.set_min_value("0");
	//	max_alarm	not set for TuneMin
	//	min_alarm	not set for TuneMin
	//	max_warning	not set for TuneMin
	//	min_warning	not set for TuneMin
	//	delta_t	not set for TuneMin
	//	delta_val	not set for TuneMin
	tunemin->set_default_properties(tunemin_prop);
	//	Not Polled
	tunemin->set_disp_level(Tango::OPERATOR);
	tunemin->set_memorized();
	tunemin->set_memorized_init(true);
	att_list.push_back(tunemin);

	//	Attribute : TuneMax
	TuneMaxAttrib	*tunemax = new TuneMaxAttrib();
	Tango::UserDefaultAttrProp	tunemax_prop;
	//	description	not set for TuneMax
	//	label	not set for TuneMax
	//	unit	not set for TuneMax
	//	standard_unit	not set for TuneMax
	//	display_unit	not set for TuneMax
	tunemax_prop.set_format("%.4f");
	tunemax_prop.set_max_value("1");
	tunemax_prop.set_min_value("0");
	//	max_alarm	not set for TuneMax
	//	min_alarm	not set for TuneMax
	//	max_warning	not set for TuneMax
	//	min_warning	not set for TuneMax
	//	delta_t	not set for TuneMax
	//	delta_val	not set for TuneMax
	tunemax->set_default_properties(tunemax_prop);
	//	Not Polled
	tunemax->set_disp_level(Tango::OPERATOR);
	tunemax->set_memorized();
	tunemax->set_memorized_init(true);
	att_list.push_back(tunemax);

	//	Attribute : TuneMinPeakThresh
	TuneMinPeakThreshAttrib	*tuneminpeakthresh = new TuneMinPeakThreshAttrib();
	Tango::UserDefaultAttrProp	tuneminpeakthresh_prop;
	//	description	not set for TuneMinPeakThresh
	//	label	not set for TuneMinPeakThresh
	//	unit	not set for TuneMinPeakThresh
	//	standard_unit	not set for TuneMinPeakThresh
	//	display_unit	not set for TuneMinPeakThresh
	tuneminpeakthresh_prop.set_format("%.2f");
	tuneminpeakthresh_prop.set_max_value("99999999");
	tuneminpeakthresh_prop.set_min_value("0");
	//	max_alarm	not set for TuneMinPeakThresh
	//	min_alarm	not set for TuneMinPeakThresh
	//	max_warning	not set for TuneMinPeakThresh
	//	min_warning	not set for TuneMinPeakThresh
	//	delta_t	not set for TuneMinPeakThresh
	//	delta_val	not set for TuneMinPeakThresh
	tuneminpeakthresh->set_default_properties(tuneminpeakthresh_prop);
	//	Not Polled
	tuneminpeakthresh->set_disp_level(Tango::OPERATOR);
	tuneminpeakthresh->set_memorized();
	tuneminpeakthresh->set_memorized_init(true);
	att_list.push_back(tuneminpeakthresh);

	//	Attribute : Tune
	TuneAttrib	*tune = new TuneAttrib();
	Tango::UserDefaultAttrProp	tune_prop;
	//	description	not set for Tune
	//	label	not set for Tune
	//	unit	not set for Tune
	//	standard_unit	not set for Tune
	//	display_unit	not set for Tune
	tune_prop.set_format("%.4f");
	tune_prop.set_max_value("1");
	tune_prop.set_min_value("0");
	//	max_alarm	not set for Tune
	//	min_alarm	not set for Tune
	//	max_warning	not set for Tune
	//	min_warning	not set for Tune
	//	delta_t	not set for Tune
	//	delta_val	not set for Tune
	tune->set_default_properties(tune_prop);
	//	Not Polled
	tune->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(tune);

	//	Attribute : MaxNumBpmUsed
	MaxNumBpmUsedAttrib	*maxnumbpmused = new MaxNumBpmUsedAttrib();
	Tango::UserDefaultAttrProp	maxnumbpmused_prop;
	maxnumbpmused_prop.set_description("Number of bpms used for tune estimation");
	//	label	not set for MaxNumBpmUsed
	//	unit	not set for MaxNumBpmUsed
	//	standard_unit	not set for MaxNumBpmUsed
	//	display_unit	not set for MaxNumBpmUsed
	maxnumbpmused_prop.set_format("%d");
	maxnumbpmused_prop.set_max_value("299");
	maxnumbpmused_prop.set_min_value("1");
	//	max_alarm	not set for MaxNumBpmUsed
	//	min_alarm	not set for MaxNumBpmUsed
	//	max_warning	not set for MaxNumBpmUsed
	//	min_warning	not set for MaxNumBpmUsed
	//	delta_t	not set for MaxNumBpmUsed
	//	delta_val	not set for MaxNumBpmUsed
	maxnumbpmused->set_default_properties(maxnumbpmused_prop);
	//	Not Polled
	maxnumbpmused->set_disp_level(Tango::OPERATOR);
	maxnumbpmused->set_memorized();
	maxnumbpmused->set_memorized_init(true);
	att_list.push_back(maxnumbpmused);

	//	Attribute : LPCoeff
	LPCoeffAttrib	*lpcoeff = new LPCoeffAttrib();
	Tango::UserDefaultAttrProp	lpcoeff_prop;
	//	description	not set for LPCoeff
	//	label	not set for LPCoeff
	//	unit	not set for LPCoeff
	//	standard_unit	not set for LPCoeff
	//	display_unit	not set for LPCoeff
	lpcoeff_prop.set_format("%.5f");
	lpcoeff_prop.set_max_value("1");
	lpcoeff_prop.set_min_value("0");
	//	max_alarm	not set for LPCoeff
	//	min_alarm	not set for LPCoeff
	//	max_warning	not set for LPCoeff
	//	min_warning	not set for LPCoeff
	//	delta_t	not set for LPCoeff
	//	delta_val	not set for LPCoeff
	lpcoeff->set_default_properties(lpcoeff_prop);
	//	Not Polled
	lpcoeff->set_disp_level(Tango::OPERATOR);
	lpcoeff->set_memorized();
	lpcoeff->set_memorized_init(true);
	att_list.push_back(lpcoeff);

	//	Attribute : TuneNotch
	TuneNotchAttrib	*tunenotch = new TuneNotchAttrib();
	Tango::UserDefaultAttrProp	tunenotch_prop;
	//	description	not set for TuneNotch
	//	label	not set for TuneNotch
	//	unit	not set for TuneNotch
	//	standard_unit	not set for TuneNotch
	//	display_unit	not set for TuneNotch
	tunenotch_prop.set_format("%.4f");
	tunenotch_prop.set_max_value("0.5");
	tunenotch_prop.set_min_value("0");
	//	max_alarm	not set for TuneNotch
	//	min_alarm	not set for TuneNotch
	//	max_warning	not set for TuneNotch
	//	min_warning	not set for TuneNotch
	//	delta_t	not set for TuneNotch
	//	delta_val	not set for TuneNotch
	tunenotch->set_default_properties(tunenotch_prop);
	//	Not Polled
	tunenotch->set_disp_level(Tango::OPERATOR);
	tunenotch->set_memorized();
	tunenotch->set_memorized_init(true);
	att_list.push_back(tunenotch);

	//	Attribute : TuneNotchWidth
	TuneNotchWidthAttrib	*tunenotchwidth = new TuneNotchWidthAttrib();
	Tango::UserDefaultAttrProp	tunenotchwidth_prop;
	//	description	not set for TuneNotchWidth
	//	label	not set for TuneNotchWidth
	//	unit	not set for TuneNotchWidth
	//	standard_unit	not set for TuneNotchWidth
	//	display_unit	not set for TuneNotchWidth
	tunenotchwidth_prop.set_format("%.4f");
	tunenotchwidth_prop.set_max_value("0.5");
	tunenotchwidth_prop.set_min_value("0");
	//	max_alarm	not set for TuneNotchWidth
	//	min_alarm	not set for TuneNotchWidth
	//	max_warning	not set for TuneNotchWidth
	//	min_warning	not set for TuneNotchWidth
	//	delta_t	not set for TuneNotchWidth
	//	delta_val	not set for TuneNotchWidth
	tunenotchwidth->set_default_properties(tunenotchwidth_prop);
	//	Not Polled
	tunenotchwidth->set_disp_level(Tango::OPERATOR);
	tunenotchwidth->set_memorized();
	tunenotchwidth->set_memorized_init(true);
	att_list.push_back(tunenotchwidth);

	//	Attribute : InhibitCycles
	InhibitCyclesAttrib	*inhibitcycles = new InhibitCyclesAttrib();
	Tango::UserDefaultAttrProp	inhibitcycles_prop;
	//	description	not set for InhibitCycles
	//	label	not set for InhibitCycles
	//	unit	not set for InhibitCycles
	//	standard_unit	not set for InhibitCycles
	//	display_unit	not set for InhibitCycles
	inhibitcycles_prop.set_format("%d");
	inhibitcycles_prop.set_max_value("999");
	inhibitcycles_prop.set_min_value("0");
	//	max_alarm	not set for InhibitCycles
	//	min_alarm	not set for InhibitCycles
	//	max_warning	not set for InhibitCycles
	//	min_warning	not set for InhibitCycles
	//	delta_t	not set for InhibitCycles
	//	delta_val	not set for InhibitCycles
	inhibitcycles->set_default_properties(inhibitcycles_prop);
	//	Not Polled
	inhibitcycles->set_disp_level(Tango::OPERATOR);
	inhibitcycles->set_memorized();
	inhibitcycles->set_memorized_init(true);
	att_list.push_back(inhibitcycles);

	//	Attribute : InhibitSegmentSize
	InhibitSegmentSizeAttrib	*inhibitsegmentsize = new InhibitSegmentSizeAttrib();
	Tango::UserDefaultAttrProp	inhibitsegmentsize_prop;
	//	description	not set for InhibitSegmentSize
	//	label	not set for InhibitSegmentSize
	//	unit	not set for InhibitSegmentSize
	//	standard_unit	not set for InhibitSegmentSize
	//	display_unit	not set for InhibitSegmentSize
	inhibitsegmentsize_prop.set_format("%d");
	inhibitsegmentsize_prop.set_max_value("1000000");
	inhibitsegmentsize_prop.set_min_value("0");
	//	max_alarm	not set for InhibitSegmentSize
	//	min_alarm	not set for InhibitSegmentSize
	//	max_warning	not set for InhibitSegmentSize
	//	min_warning	not set for InhibitSegmentSize
	//	delta_t	not set for InhibitSegmentSize
	//	delta_val	not set for InhibitSegmentSize
	inhibitsegmentsize->set_default_properties(inhibitsegmentsize_prop);
	//	Not Polled
	inhibitsegmentsize->set_disp_level(Tango::OPERATOR);
	inhibitsegmentsize->set_memorized();
	inhibitsegmentsize->set_memorized_init(true);
	att_list.push_back(inhibitsegmentsize);

	//	Attribute : DebugMinThres
	DebugMinThresAttrib	*debugminthres = new DebugMinThresAttrib();
	Tango::UserDefaultAttrProp	debugminthres_prop;
	//	description	not set for DebugMinThres
	//	label	not set for DebugMinThres
	//	unit	not set for DebugMinThres
	//	standard_unit	not set for DebugMinThres
	//	display_unit	not set for DebugMinThres
	debugminthres_prop.set_format("%f");
	debugminthres_prop.set_max_value("0.5");
	debugminthres_prop.set_min_value("0");
	//	max_alarm	not set for DebugMinThres
	//	min_alarm	not set for DebugMinThres
	//	max_warning	not set for DebugMinThres
	//	min_warning	not set for DebugMinThres
	//	delta_t	not set for DebugMinThres
	//	delta_val	not set for DebugMinThres
	debugminthres->set_default_properties(debugminthres_prop);
	//	Not Polled
	debugminthres->set_disp_level(Tango::OPERATOR);
	debugminthres->set_memorized();
	debugminthres->set_memorized_init(true);
	att_list.push_back(debugminthres);

	//	Attribute : DebugMaxThres
	DebugMaxThresAttrib	*debugmaxthres = new DebugMaxThresAttrib();
	Tango::UserDefaultAttrProp	debugmaxthres_prop;
	//	description	not set for DebugMaxThres
	//	label	not set for DebugMaxThres
	//	unit	not set for DebugMaxThres
	//	standard_unit	not set for DebugMaxThres
	//	display_unit	not set for DebugMaxThres
	debugmaxthres_prop.set_format("%f");
	debugmaxthres_prop.set_max_value("0.5");
	debugmaxthres_prop.set_min_value("0");
	//	max_alarm	not set for DebugMaxThres
	//	min_alarm	not set for DebugMaxThres
	//	max_warning	not set for DebugMaxThres
	//	min_warning	not set for DebugMaxThres
	//	delta_t	not set for DebugMaxThres
	//	delta_val	not set for DebugMaxThres
	debugmaxthres->set_default_properties(debugmaxthres_prop);
	//	Not Polled
	debugmaxthres->set_disp_level(Tango::OPERATOR);
	debugmaxthres->set_memorized();
	debugmaxthres->set_memorized_init(true);
	att_list.push_back(debugmaxthres);

	//	Attribute : HistoryBufSize
	HistoryBufSizeAttrib	*historybufsize = new HistoryBufSizeAttrib();
	Tango::UserDefaultAttrProp	historybufsize_prop;
	//	description	not set for HistoryBufSize
	//	label	not set for HistoryBufSize
	//	unit	not set for HistoryBufSize
	//	standard_unit	not set for HistoryBufSize
	//	display_unit	not set for HistoryBufSize
	historybufsize_prop.set_format("%d");
	historybufsize_prop.set_max_value("2000000");
	historybufsize_prop.set_min_value("1");
	//	max_alarm	not set for HistoryBufSize
	//	min_alarm	not set for HistoryBufSize
	//	max_warning	not set for HistoryBufSize
	//	min_warning	not set for HistoryBufSize
	//	delta_t	not set for HistoryBufSize
	//	delta_val	not set for HistoryBufSize
	historybufsize->set_default_properties(historybufsize_prop);
	//	Not Polled
	historybufsize->set_disp_level(Tango::OPERATOR);
	historybufsize->set_memorized();
	historybufsize->set_memorized_init(true);
	att_list.push_back(historybufsize);

	//	Attribute : EnableDespiking
	EnableDespikingAttrib	*enabledespiking = new EnableDespikingAttrib();
	Tango::UserDefaultAttrProp	enabledespiking_prop;
	//	description	not set for EnableDespiking
	//	label	not set for EnableDespiking
	//	unit	not set for EnableDespiking
	//	standard_unit	not set for EnableDespiking
	//	display_unit	not set for EnableDespiking
	//	format	not set for EnableDespiking
	//	max_value	not set for EnableDespiking
	//	min_value	not set for EnableDespiking
	//	max_alarm	not set for EnableDespiking
	//	min_alarm	not set for EnableDespiking
	//	max_warning	not set for EnableDespiking
	//	min_warning	not set for EnableDespiking
	//	delta_t	not set for EnableDespiking
	//	delta_val	not set for EnableDespiking
	enabledespiking->set_default_properties(enabledespiking_prop);
	//	Not Polled
	enabledespiking->set_disp_level(Tango::OPERATOR);
	enabledespiking->set_memorized();
	enabledespiking->set_memorized_init(true);
	att_list.push_back(enabledespiking);

	//	Attribute : DespikingFilterLen
	DespikingFilterLenAttrib	*despikingfilterlen = new DespikingFilterLenAttrib();
	Tango::UserDefaultAttrProp	despikingfilterlen_prop;
	//	description	not set for DespikingFilterLen
	//	label	not set for DespikingFilterLen
	//	unit	not set for DespikingFilterLen
	//	standard_unit	not set for DespikingFilterLen
	//	display_unit	not set for DespikingFilterLen
	despikingfilterlen_prop.set_format("%d");
	despikingfilterlen_prop.set_max_value("99");
	despikingfilterlen_prop.set_min_value("1");
	//	max_alarm	not set for DespikingFilterLen
	//	min_alarm	not set for DespikingFilterLen
	//	max_warning	not set for DespikingFilterLen
	//	min_warning	not set for DespikingFilterLen
	//	delta_t	not set for DespikingFilterLen
	//	delta_val	not set for DespikingFilterLen
	despikingfilterlen->set_default_properties(despikingfilterlen_prop);
	//	Not Polled
	despikingfilterlen->set_disp_level(Tango::OPERATOR);
	despikingfilterlen->set_memorized();
	despikingfilterlen->set_memorized_init(true);
	att_list.push_back(despikingfilterlen);

	//	Attribute : Frequency
	FrequencyAttrib	*frequency = new FrequencyAttrib();
	Tango::UserDefaultAttrProp	frequency_prop;
	//	description	not set for Frequency
	//	label	not set for Frequency
	frequency_prop.set_unit("Hz");
	frequency_prop.set_standard_unit("Hz");
	frequency_prop.set_display_unit("Hz");
	frequency_prop.set_format("%f");
	frequency_prop.set_max_value("99999");
	frequency_prop.set_min_value("0");
	//	max_alarm	not set for Frequency
	//	min_alarm	not set for Frequency
	//	max_warning	not set for Frequency
	//	min_warning	not set for Frequency
	//	delta_t	not set for Frequency
	//	delta_val	not set for Frequency
	frequency->set_default_properties(frequency_prop);
	//	Not Polled
	frequency->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(frequency);

	//	Attribute : FrequencySet
	FrequencySetAttrib	*frequencyset = new FrequencySetAttrib();
	Tango::UserDefaultAttrProp	frequencyset_prop;
	//	description	not set for FrequencySet
	//	label	not set for FrequencySet
	frequencyset_prop.set_unit("Hz");
	frequencyset_prop.set_standard_unit("Hz");
	frequencyset_prop.set_display_unit("Hz");
	frequencyset_prop.set_format("%f");
	frequencyset_prop.set_max_value("99999");
	frequencyset_prop.set_min_value("0");
	//	max_alarm	not set for FrequencySet
	//	min_alarm	not set for FrequencySet
	//	max_warning	not set for FrequencySet
	//	min_warning	not set for FrequencySet
	//	delta_t	not set for FrequencySet
	//	delta_val	not set for FrequencySet
	frequencyset->set_default_properties(frequencyset_prop);
	//	Not Polled
	frequencyset->set_disp_level(Tango::OPERATOR);
	frequencyset->set_memorized();
	frequencyset->set_memorized_init(true);
	att_list.push_back(frequencyset);

	//	Attribute : InjTune
	InjTuneAttrib	*injtune = new InjTuneAttrib();
	Tango::UserDefaultAttrProp	injtune_prop;
	//	description	not set for InjTune
	//	label	not set for InjTune
	//	unit	not set for InjTune
	//	standard_unit	not set for InjTune
	//	display_unit	not set for InjTune
	injtune_prop.set_format("%.4f");
	injtune_prop.set_max_value("1");
	injtune_prop.set_min_value("0");
	//	max_alarm	not set for InjTune
	//	min_alarm	not set for InjTune
	//	max_warning	not set for InjTune
	//	min_warning	not set for InjTune
	//	delta_t	not set for InjTune
	//	delta_val	not set for InjTune
	injtune->set_default_properties(injtune_prop);
	//	Not Polled
	injtune->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(injtune);

	//	Attribute : InjPeakDetectThresh
	InjPeakDetectThreshAttrib	*injpeakdetectthresh = new InjPeakDetectThreshAttrib();
	Tango::UserDefaultAttrProp	injpeakdetectthresh_prop;
	//	description	not set for InjPeakDetectThresh
	//	label	not set for InjPeakDetectThresh
	injpeakdetectthresh_prop.set_unit("mm");
	//	standard_unit	not set for InjPeakDetectThresh
	injpeakdetectthresh_prop.set_display_unit("mm");
	injpeakdetectthresh_prop.set_format("%f");
	injpeakdetectthresh_prop.set_max_value("99");
	injpeakdetectthresh_prop.set_min_value("0");
	//	max_alarm	not set for InjPeakDetectThresh
	//	min_alarm	not set for InjPeakDetectThresh
	//	max_warning	not set for InjPeakDetectThresh
	//	min_warning	not set for InjPeakDetectThresh
	//	delta_t	not set for InjPeakDetectThresh
	//	delta_val	not set for InjPeakDetectThresh
	injpeakdetectthresh->set_default_properties(injpeakdetectthresh_prop);
	//	Not Polled
	injpeakdetectthresh->set_disp_level(Tango::OPERATOR);
	injpeakdetectthresh->set_memorized();
	injpeakdetectthresh->set_memorized_init(true);
	att_list.push_back(injpeakdetectthresh);

	//	Attribute : SumThres
	SumThresAttrib	*sumthres = new SumThresAttrib();
	Tango::UserDefaultAttrProp	sumthres_prop;
	//	description	not set for SumThres
	//	label	not set for SumThres
	sumthres_prop.set_unit("a.u.");
	//	standard_unit	not set for SumThres
	sumthres_prop.set_display_unit("a.u.");
	sumthres_prop.set_format("%d");
	sumthres_prop.set_max_value("4000000000");
	sumthres_prop.set_min_value("0");
	//	max_alarm	not set for SumThres
	//	min_alarm	not set for SumThres
	//	max_warning	not set for SumThres
	//	min_warning	not set for SumThres
	//	delta_t	not set for SumThres
	//	delta_val	not set for SumThres
	sumthres->set_default_properties(sumthres_prop);
	//	Not Polled
	sumthres->set_disp_level(Tango::OPERATOR);
	sumthres->set_memorized();
	sumthres->set_memorized_init(true);
	att_list.push_back(sumthres);

	//	Attribute : EnableInjDespiking
	EnableInjDespikingAttrib	*enableinjdespiking = new EnableInjDespikingAttrib();
	Tango::UserDefaultAttrProp	enableinjdespiking_prop;
	//	description	not set for EnableInjDespiking
	//	label	not set for EnableInjDespiking
	//	unit	not set for EnableInjDespiking
	//	standard_unit	not set for EnableInjDespiking
	//	display_unit	not set for EnableInjDespiking
	//	format	not set for EnableInjDespiking
	//	max_value	not set for EnableInjDespiking
	//	min_value	not set for EnableInjDespiking
	//	max_alarm	not set for EnableInjDespiking
	//	min_alarm	not set for EnableInjDespiking
	//	max_warning	not set for EnableInjDespiking
	//	min_warning	not set for EnableInjDespiking
	//	delta_t	not set for EnableInjDespiking
	//	delta_val	not set for EnableInjDespiking
	enableinjdespiking->set_default_properties(enableinjdespiking_prop);
	//	Not Polled
	enableinjdespiking->set_disp_level(Tango::OPERATOR);
	enableinjdespiking->set_memorized();
	enableinjdespiking->set_memorized_init(true);
	att_list.push_back(enableinjdespiking);

	//	Attribute : TunePeakAmplitude
	TunePeakAmplitudeAttrib	*tunepeakamplitude = new TunePeakAmplitudeAttrib();
	Tango::UserDefaultAttrProp	tunepeakamplitude_prop;
	//	description	not set for TunePeakAmplitude
	//	label	not set for TunePeakAmplitude
	//	unit	not set for TunePeakAmplitude
	//	standard_unit	not set for TunePeakAmplitude
	//	display_unit	not set for TunePeakAmplitude
	tunepeakamplitude_prop.set_format("%.2f");
	tunepeakamplitude_prop.set_max_value("1e10");
	tunepeakamplitude_prop.set_min_value("0");
	//	max_alarm	not set for TunePeakAmplitude
	//	min_alarm	not set for TunePeakAmplitude
	//	max_warning	not set for TunePeakAmplitude
	//	min_warning	not set for TunePeakAmplitude
	//	delta_t	not set for TunePeakAmplitude
	//	delta_val	not set for TunePeakAmplitude
	tunepeakamplitude->set_default_properties(tunepeakamplitude_prop);
	//	Not Polled
	tunepeakamplitude->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(tunepeakamplitude);

	//	Attribute : FFTRaw
	FFTRawAttrib	*fftraw = new FFTRawAttrib();
	Tango::UserDefaultAttrProp	fftraw_prop;
	//	description	not set for FFTRaw
	//	label	not set for FFTRaw
	//	unit	not set for FFTRaw
	//	standard_unit	not set for FFTRaw
	//	display_unit	not set for FFTRaw
	fftraw_prop.set_format("%.4f");
	fftraw_prop.set_max_value("999999999");
	fftraw_prop.set_min_value("0");
	//	max_alarm	not set for FFTRaw
	//	min_alarm	not set for FFTRaw
	//	max_warning	not set for FFTRaw
	//	min_warning	not set for FFTRaw
	//	delta_t	not set for FFTRaw
	//	delta_val	not set for FFTRaw
	fftraw->set_default_properties(fftraw_prop);
	//	Not Polled
	fftraw->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(fftraw);

	//	Attribute : FFTFilt
	FFTFiltAttrib	*fftfilt = new FFTFiltAttrib();
	Tango::UserDefaultAttrProp	fftfilt_prop;
	//	description	not set for FFTFilt
	//	label	not set for FFTFilt
	//	unit	not set for FFTFilt
	//	standard_unit	not set for FFTFilt
	//	display_unit	not set for FFTFilt
	fftfilt_prop.set_format("%.4f");
	fftfilt_prop.set_max_value("999999999");
	fftfilt_prop.set_min_value("0");
	//	max_alarm	not set for FFTFilt
	//	min_alarm	not set for FFTFilt
	//	max_warning	not set for FFTFilt
	//	min_warning	not set for FFTFilt
	//	delta_t	not set for FFTFilt
	//	delta_val	not set for FFTFilt
	fftfilt->set_default_properties(fftfilt_prop);
	//	Not Polled
	fftfilt->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(fftfilt);

	//	Attribute : BpmIndexList
	BpmIndexListAttrib	*bpmindexlist = new BpmIndexListAttrib();
	Tango::UserDefaultAttrProp	bpmindexlist_prop;
	//	description	not set for BpmIndexList
	//	label	not set for BpmIndexList
	//	unit	not set for BpmIndexList
	//	standard_unit	not set for BpmIndexList
	//	display_unit	not set for BpmIndexList
	//	format	not set for BpmIndexList
	//	max_value	not set for BpmIndexList
	//	min_value	not set for BpmIndexList
	//	max_alarm	not set for BpmIndexList
	//	min_alarm	not set for BpmIndexList
	//	max_warning	not set for BpmIndexList
	//	min_warning	not set for BpmIndexList
	//	delta_t	not set for BpmIndexList
	//	delta_val	not set for BpmIndexList
	bpmindexlist->set_default_properties(bpmindexlist_prop);
	//	Not Polled
	bpmindexlist->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(bpmindexlist);

	//	Attribute : TuneHistory
	TuneHistoryAttrib	*tunehistory = new TuneHistoryAttrib();
	Tango::UserDefaultAttrProp	tunehistory_prop;
	//	description	not set for TuneHistory
	//	label	not set for TuneHistory
	//	unit	not set for TuneHistory
	//	standard_unit	not set for TuneHistory
	//	display_unit	not set for TuneHistory
	tunehistory_prop.set_format("%.4f");
	tunehistory_prop.set_max_value("0.5");
	tunehistory_prop.set_min_value("0");
	//	max_alarm	not set for TuneHistory
	//	min_alarm	not set for TuneHistory
	//	max_warning	not set for TuneHistory
	//	min_warning	not set for TuneHistory
	//	delta_t	not set for TuneHistory
	//	delta_val	not set for TuneHistory
	tunehistory->set_default_properties(tunehistory_prop);
	//	Not Polled
	tunehistory->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(tunehistory);

	//	Attribute : Debug
	DebugAttrib	*debug = new DebugAttrib();
	Tango::UserDefaultAttrProp	debug_prop;
	//	description	not set for Debug
	//	label	not set for Debug
	//	unit	not set for Debug
	//	standard_unit	not set for Debug
	//	display_unit	not set for Debug
	//	format	not set for Debug
	//	max_value	not set for Debug
	//	min_value	not set for Debug
	//	max_alarm	not set for Debug
	//	min_alarm	not set for Debug
	//	max_warning	not set for Debug
	//	min_warning	not set for Debug
	//	delta_t	not set for Debug
	//	delta_val	not set for Debug
	debug->set_default_properties(debug_prop);
	//	Not Polled
	debug->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(debug);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(TuneGof2Class::attribute_factory_after) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::pipe_factory()
 * Description:  Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void TuneGof2Class::pipe_factory()
{
	/*----- PROTECTED REGION ID(TuneGof2Class::pipe_factory_before) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::pipe_factory_before
	/*----- PROTECTED REGION ID(TuneGof2Class::pipe_factory_after) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::command_factory()
 * Description:  Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void TuneGof2Class::command_factory()
{
	/*----- PROTECTED REGION ID(TuneGof2Class::command_factory_before) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::command_factory_before


	//	Command Start
	StartClass	*pStartCmd =
		new StartClass("Start",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pStartCmd);

	//	Command Stop
	StopClass	*pStopCmd =
		new StopClass("Stop",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pStopCmd);

	//	Command GetTune
	GetTuneClass	*pGetTuneCmd =
		new GetTuneClass("GetTune",
			Tango::DEVVAR_LONGARRAY, Tango::DEVVAR_FLOATARRAY,
			"mode(0);num samples",
			"",
			Tango::OPERATOR);
	command_list.push_back(pGetTuneCmd);

	/*----- PROTECTED REGION ID(TuneGof2Class::command_factory_after) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		TuneGof2Class::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the created attribute list
 */
//--------------------------------------------------------
void TuneGof2Class::create_static_attribute_list(std::vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		std::string att_name(att_list[i]->get_name());
		transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	TANGO_LOG_INFO << defaultAttList.size() << " attributes in default list" << std::endl;

	/*----- PROTECTED REGION ID(TuneGof2Class::create_static_att_list) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		TuneGof2Class::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void TuneGof2Class::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, std::vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((std::string)(*devlist_ptr)[i]).c_str());
		TuneGof2 *dev = static_cast<TuneGof2 *> (dev_impl);

		std::vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		std::vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			std::string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			std::vector<std::string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				TANGO_LOG_INFO << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << std::endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(TuneGof2Class::erase_dynamic_attributes) ENABLED START -----*/
	/* clang-format on */
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method      : TuneGof2Class::get_attr_object_by_name()
 * Description:  returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *TuneGof2Class::get_attr_object_by_name(std::vector<Tango::Attr *> &att_list, std::string attname)
{
	std::vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(TuneGof2Class::Additional Methods) ENABLED START -----*/
/* clang-format on */
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	TuneGof2Class::Additional Methods
} //	namespace
