/*
 * tunethread.cpp
 */


#include "tunethread.h"



/* 
 * tunethread::run()
 * Run  
 */		
void tunethread::run(void *)
{
	struct timespec now, old, expire;
	double avg, avg_old, avg_consec = 0;

	avg_old = 0;
	avg = 0;

	myobj->run_tune_flag = true;

	Tango::DevVarULongArray *argin = new Tango::DevVarULongArray();
	argin->length(5);

	(*argin)[1] = 1; // one parameter

	if (myobj->plane == "Horizontal")
		(*argin)[2] = 6; // vertical position	
	else if (myobj->plane == "Vertical")
		(*argin)[2] = 7; // vertical position	
	else 
		myobj->abort_tune_flag = true;
	
	(*argin)[3] = myobj->bpmIndexList[0]; // lgbpm_s7.1	
	(*argin)[4] = myobj->bpmIndexList[1]; // lgbpm_s7.1	

	clock_gettime(CLOCK_MONOTONIC, &now);
	clock_gettime(CLOCK_MONOTONIC, &expire);

	while (myobj->abort_tune_flag == false) {

		int int_peak_found[2], ext_peak_found[2], sum_err[2];

		double period_seconds = 1.0 / *myobj->attr_FrequencySet_read;
		// Converte il periodo in secondi e nanosecondi
		long seconds = (long)period_seconds;
		long nanoseconds = (long)((period_seconds - seconds) * 1e9);
		struct timespec req = {seconds, nanoseconds};

		expire = myobj->add_timespec(expire, req);

		(*argin)[0] = *myobj->attr_NumSamples_read;

		myobj->get_bpm_fftdata(argin, int_peak_found, ext_peak_found, sum_err);

		int fft_size = *myobj->attr_NumSamples_read/2;
		
		int start_idx = (double)*myobj->attr_NumSamples_read * *myobj->attr_TuneMin_read;
		int end_idx = (double)*myobj->attr_NumSamples_read * *myobj->attr_TuneMax_read;

		// add more samples for the filtering
		int start_idx_zero = start_idx-TUNEGOF2_LPF_RESPONSE_TIME;
		if (start_idx_zero < 0)
			start_idx_zero = 0;

		int end_idx_zero = end_idx+TUNEGOF2_LPF_RESPONSE_TIME;
		if (end_idx_zero >= fft_size)
			end_idx_zero = fft_size;

		for (int k = 0; k < fft_size; k++) { // num_samples	
			myobj->tune_fft_raw_data[k] = 1;
			for (int j = 0; j < 2; j++) {
				myobj->tune_fft_raw_data[k] *= myobj->tune_fft_bpm_data[j][k];
			}
			// force element outside the fft (expanded) window to zero
			if ((k < start_idx_zero) || (k >= end_idx_zero))
				myobj->tune_fft_raw_data[k] = 0;
		}

		// low pass filter, zero phase shift
		myobj->filtfilt(myobj->tune_fft_raw_data,myobj->tune_fft_filt_data,myobj->filtbuf_tmp,fft_size);
	
		double max_val = 0;
		int max_idx = 0;

		avg_old = avg;
		avg = 0;

		for (int k = start_idx; k < end_idx; k++) {
			if (myobj->tune_fft_filt_data[k] > max_val) {
				max_val = myobj->tune_fft_filt_data[k];
				max_idx = k;		
			}
			avg += myobj->tune_fft_filt_data[k];
		}
		
		avg /= (double) (end_idx-start_idx);

		*myobj->attr_TunePeakAmplitude_read = (max_val - avg);
	
		if (avg_old == avg)
			avg_consec++;
		else
			avg_consec = 0;

		if (avg_consec > 5) {
			myobj->set_state(Tango::FAULT);
			myobj->set_status("FAULT, bpm data freezed");
		}
		else if (*myobj->attr_TunePeakAmplitude_read < *myobj->attr_TuneMinPeakThresh_read) {
			myobj->set_state(Tango::FAULT);
			myobj->set_status("FAULT, tune peak amplitude under TuneMinPeakThresh");
		}
		else if ((sum_err[0] >= 100) || (sum_err[1] >= 100)) {
			myobj->set_state(Tango::FAULT);
			myobj->set_status("FAULT, signal under threshold");
		}
		else {
			myobj->set_state(Tango::ON);
			myobj->set_status("Tune measurement ON");			
		}

		*myobj->attr_Tune_read = (double)max_idx / (double)*myobj->attr_NumSamples_read;

		

		if (int_peak_found[0] * int_peak_found[1]) {
			*myobj->attr_InjTune_read = *myobj->attr_Tune_read;
		}

		myobj->tune_circ_buf[myobj->tune_buf_idx] = (double)max_idx / (double)*myobj->attr_NumSamples_read;

		int desp_len = *myobj->attr_DespikingFilterLen_read; // to avoid free corruption
		int hist_buf_size = *myobj->attr_HistoryBufSize_read;

		// if despiking is enable the tune value is the median of the last *attr_DespikingFilterLen_read samples
		if (*myobj->attr_EnableDespiking_read) {
			vector <float> last_values;
			for (int i = 0; i < desp_len; i++) {
				last_values.push_back(myobj->tune_circ_buf[(myobj->tune_buf_idx-i+hist_buf_size) % hist_buf_size]);
			}
			*myobj->attr_Tune_read = myobj->median(last_values);
		}

		if ((myobj->tune_circ_buf[myobj->tune_buf_idx] > *myobj->attr_DebugMaxThres_read) ||
			(myobj->tune_circ_buf[myobj->tune_buf_idx] < *myobj->attr_DebugMinThres_read))
			myobj->abort_tune_flag = true;

		if (++myobj->tune_buf_idx >= hist_buf_size)
			myobj->tune_buf_idx = 0;



		if (myobj->abort_tune_flag)
			break;

		// wait the time to make FrequencySet as close as Frequency
		while (1) {
			clock_gettime(CLOCK_MONOTONIC, &now);
			long diff_time = myobj->timespec_to_nsec(myobj->subtract_timespec(expire, now));				
			if (diff_time < 5000) { // ns. half of the sleeping time		
				break;
			}
			if (diff_time > 3000000000) {		
				expire.tv_sec = now.tv_sec; expire.tv_nsec = now.tv_nsec;
			}			
			usleep(10);
		}

		long diff_time = myobj->timespec_to_nsec(myobj->subtract_timespec(now, old));
		old.tv_sec = now.tv_sec; old.tv_nsec = now.tv_nsec;
		*myobj->attr_Frequency_read = *myobj->attr_Frequency_read * 0.99 + 0.01 * 1000000000 / (double)diff_time; 
		
	}

	myobj->set_state(Tango::OFF);
	myobj->set_status("Tune measumerent OFF");

	delete argin;

	myobj->abort_tune_flag = false;
	myobj->run_tune_flag = false;
	
} /* end rmthread::run() */	
	

